/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2016 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.inbox.ui;

import java.util.Date;
import java.util.Map;
import java.util.Set;

import aQute.bnd.annotation.ProviderType;

/**
 * Represents an inbox item of either type task or workitem.
 */
@ProviderType
public interface InboxItem {

    /**
     * Returns the item title.
     * 
     * @return item title
     */
    String getTitle() {
		return null;
	}

    /**
     * Returns the item description.
     * 
     * @return item description 
     */
    String getDescription() {
		return null;
	}

    /**
     * Returns the item path.
     * 
     * @return item path
     */
    String getPath() {
		return null;
	}

    /**
     * Returns the item resource type.
     * 
     * @return item resource type
     */
    String getResourceType() {
		return null;
	}

    /**
     * Returns a Map containing information about the current assignee.
     * 
     * @return a map of key/value pairs or {@code null}
     */
    Map<String, String> getCurrentAssigneeInfo() {
		return null;
	}

    /**
     * Returns the payload link.
     * 
     * @return payload link or {@code null}
     */
    String getPayloadLink() {
		return null;
	}

    /**
     * Returns the payload thumbnail path. If the payload has no payload thumbnail,
     * {@code null} is returned.
     * 
     * @return payload thumbnail path or {@code null}
     */
    String getPayloadThumbnail() {
		return null;
	}

    /**
     * Indicates if referenced payload is of type folder.
     * 
     * @return {@code true} if payload is of type folder, {@code false} otherwise
     */
    boolean isPayloadFolder() {
		return false;
	}

    /**
     * Returns the thumbnail path. If no project is associated with this task {@code null} is returned.
     * 
     * @return thumbnail path or {@code null}
     */
    String getThumbnail() {
		return null;
	}

    /**
     * Returns the task priority. Either 'High', 'Medium' or 'Low'.
     * 
     * @return task priority or {@code null}
     */
    String getPriority() {
		return null;
	}

    /**
     * Returns the task priority icon identifier. Either 'arrowUp' or 'arrowDown'.
     * 
     * @return task priority icon identifier or {@code null}
     */
    String getPriorityIcon() {
		return null;
	}

    /**
     * Returns the task priority style class name. Either
     * 'inbox-item-priority--high', 'inbox-item-priority--normal' or
     * 'inbox-item-priority--low'.
     * 
     * @return task priority style class name
     */
    String getPriorityStyle() {
		return null;
	}

    /**
     * Returns the task status. Either 'Active', 'Complete' or 'Terminated'.
     * 
     * @return task status
     */
    String getStatus() {
		return null;
	}

    /**
     * Returns the task status icon. Either 'clock', 'checkCircle' or 'alert'.
     * 
     * @return task status icon name
     */
    String getStatusIcon() {
		return null;
	}

    /**
     * Returns the task status type. Either 'inbox-status-active' or 'inbox-status-complete'.
     * 
     * @return task status type or an empty String
     */
    String getStatusType() {
		return null;
	}

    /**
     * Returns the task details URL.
     * 
     * @return task details URL
     */
    String getDetailsUrl() {
		return null;
	}

    /**
     * Returns the task project identifier if task is related to a project.
     * 
     * @return project identifier or {@code null}
     */
    String getProjectId() {
		return null;
	}

    /**
     * Returns the task project title if task is related to a project.
     * 
     * @return task project title or {@code null}
     */
    String getProjectTitle() {
		return null;
	}

    /**
     * Returns the task project URL if task is related to a project.
     * 
     * @return task project URL or {@code null}
     */
    String getAssociatedProjectDetailsUrl() {
		return null;
	}

    /**
     * Returns the workflow title if task is related to a workflow.
     * 
     * @return task workflow title or {@code null}
     */
    String getWorkflowTitle() {
		return null;
	}

    /**
     * Returns the workflow initiator identifier if task is related to a workflow.
     * 
     * @return workflow initiator identifier or {@code null}
     */
    String getWorkflowInitiator() {
		return null;
	}

    /**
     * Returns the task start date.
     * 
     * @return task start date
     */
    Date getStartDate() {
		return null;
	}

    /**
     * Returns the task due date.
     * 
     * @return task due date
     */
    Date getDueDate() {
		return null;
	}

    /**
     * Returns the due date color name. Either 'red' or 'orange'.
     * 
     * @return due date color name or {@code null}
     */
    String getDueDateColor() {
		return null;
	}

    /**
     * Returns the quickaction relation identifiers.
     * 
     * @return quickaction relation itendifiers or an empty Set.
     */
    Set<String> getQuickactionsRel() {
		return null;
	}

    /**
     * Returns the task actions as serialized JSONArray in the following form.
     * 
     * <p>
     *  <code>
     *  [{actionName:'Action', actionId:'ID'},...]
     *  </code>
     * </p>
     * 
     * @return task actions
     */
    String getTaskActions() {
		return null;
	}

}
