/*******************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 ******************************************************************************/
package com.adobe.cq.sightly;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import javax.servlet.RequestDispatcher;
import javax.servlet.Servlet;

import org.apache.commons.lang3.StringUtils;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.SlingHttpServletResponse;
import org.apache.sling.api.request.RequestDispatcherOptions;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.SyntheticResource;
import org.apache.sling.api.resource.ValueMap;
import org.apache.sling.api.scripting.SlingScriptHelper;
import org.apache.sling.api.servlets.ServletResolver;
import org.apache.sling.api.wrappers.ValueMapDecorator;
import org.apache.sling.scripting.sightly.SightlyException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import aQute.bnd.annotation.ProviderType;
import com.adobe.cq.sightly.internal.PrintWriterResponseWrapper;
import com.day.cq.wcm.api.WCMMode;
import com.day.cq.wcm.api.components.ComponentContext;
import com.day.cq.wcm.api.components.IncludeOptions;
import com.day.cq.wcm.commons.WCMUtils;

/**
 * The {@code WCMScriptHelper} class provides helper methods to support CQ Sightly extensions. This is to be used only with the CQ specific
 * Sightly plugins.
 *
 * This class is not meant to be extended.
 **/
@ProviderType
public class WCMScriptHelper {

    private final Logger log = LoggerFactory.getLogger(WCMScriptHelper.class);

    private SlingScriptHelper slingHelper;

    private SlingHttpServletRequest request;

    private SlingHttpServletResponse response;

    private SightlyWCMMode mode;

    public WCMScriptHelper(SlingScriptHelper sling) {
        slingHelper = sling;
        request = sling.getRequest();
        response = sling.getResponse();
        mode = new SightlyWCMMode(request);
    }

    /**
     * Include the resource and redirect the output into a custom print writer.
     *
     * @param out                the custom writer
     * @param script             the path of the resource to include
     * @param dispatcherOptions  {@code key=value} comma separated pairs as String
     * @param resourceType       Sling resource type to be used while including the resource
     * @param wcmResourceOptions the WCM specific options for including the resource
     */
    public void includeResource(PrintWriter out, String script, String dispatcherOptions, String resourceType,
                                WCMResourceOptions wcmResourceOptions) {
								}

    /**
     * Include a resource and redirect the output into a custom response.
     *
     * @param customResponse     the custom response
     * @param script             the path of the resource to include
     * @param dispatcherOptions  {@code key=value} comma separated pairs as String
     * @param resourceType       Sling resource type to be used while including the resource
     * @param wcmResourceOptions the WCM specific options for including the resource
     */
    public void includeResource(SlingHttpServletResponse customResponse, String script, String dispatcherOptions, String resourceType,
                                WCMResourceOptions wcmResourceOptions) {
								}

    /**
     * Include a {@link SyntheticResource} and redirect the output into a custom response.
     *
     * @param response           the custom response
     * @param script             the path of the resource to include
     * @param dispatcherOptions  {@code key=value} comma separated pairs as String
     * @param resourceType       Sling resource type to be used while including the resource
     * @param wcmResourceOptions the WCM specific options for including the resource
     * @param resourceProperties the properties of the {@link SyntheticResource} that will be included
     */
    public void includeResource(SlingHttpServletResponse response, String script, String dispatcherOptions, String resourceType,
                                WCMResourceOptions wcmResourceOptions, Map<String, Object> resourceProperties) {
								}

    /**
     * Include a {@link Resource} and redirect the output into a custom response
     *
     * @param response           the custom response
     * @param resource           the {@link Resource} to include
     * @param dispatcherOptions  {@code key=value} comma separated pairs as String
     * @param resourceType       Sling resource type to be used while including the resource
     * @param wcmResourceOptions the WCM specific options for including the resource
     */
    public void includeResource(SlingHttpServletResponse response, Resource resource, String dispatcherOptions, String resourceType,
                                WCMResourceOptions wcmResourceOptions) {
								}

    /**
     * Calls the given script.
     *
     * @param script    script path to be called
     * @param wcmMode   WCM mode to be used, its an optional parameter. If the current request contains mode attribute,
     *                  it will be reset after the script call.
     *
     * */
    public void includeScript(String script, String wcmMode, PrintWriter out) {
	}

    public SightlyWCMMode getMode() {
		return null;
	}

    //------------------------- private ---------------------------
    private String normalizePath(String path) {
		return null;
	}

    private List<String> expandClassName(String classesString) {
		return null;
	}

    /**
     * The {@code SyntheticResourceWithProperties} allows creating virtual resources that can be used for rendering content not obeying
     * an AEM component's expected content structure.
     */
    private static class SyntheticResourceWithProperties extends SyntheticResource {

        private Map<String, Object> properties;

        SyntheticResourceWithProperties(ResourceResolver resourceResolver, String path, String resourceType, Map<String, Object> properties) {
            super(resourceResolver, path, resourceType);
            this.properties = properties;
        }

        @Override
        public <AdapterType> AdapterType adaptTo(Class<AdapterType> type) {
			return null;
		}
    }

}
