/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.activitystreams.listener.api;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.commons.lang3.StringUtils;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.ReferenceCardinality;
import org.apache.felix.scr.annotations.ReferencePolicy;
import org.apache.felix.scr.annotations.References;
import org.apache.sling.api.resource.LoginException;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceResolverFactory;
import org.apache.sling.jcr.api.SlingRepository;
import org.apache.sling.jcr.resource.JcrResourceConstants;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.activitystreams.api.SocialActivityManager;
import com.adobe.cq.social.activitystreams.impl.Activator;
import com.adobe.cq.social.activitystreams.listener.impl.ComparableStreamProviderExtension;
import com.adobe.cq.social.scf.core.SocialEvent;
import com.adobe.granite.activitystreams.Activity;
import com.adobe.granite.activitystreams.ActivityException;
import com.adobe.granite.activitystreams.ActivityStream;

/**
 * Base class of the ActivityStreamProvider.
 */
@Component(componentAbstract = true, label = "%abstractactivitystreamprovider.name")
@References({
    @Reference(name = "extensionProvider", referenceInterface = ActivityStreamProviderExtension.class,
            bind = "bindExtension", unbind = "unbindExtension", cardinality = ReferenceCardinality.OPTIONAL_MULTIPLE,
            policy = ReferencePolicy.DYNAMIC),
    @Reference(name = "listenerExtension", referenceInterface = EventListenerExtension.class,
            bind = "bindListenerExtension", unbind = "unbindListenerExtension",
            cardinality = ReferenceCardinality.OPTIONAL_MULTIPLE, policy = ReferencePolicy.DYNAMIC)})
public abstract class AbstractActivityStreamProvider implements ActivityStreamProvider {
    protected static final String UGC_WRITER = "ugc-writer";

    /**
     * Eventually remove SLING_SERVICE_SUBSERVICE once we can update the dependencies to get this from
     * ResourceResolverFactory and not make the unit tests fail.
     */
    protected static final String SLING_SERVICE_SUBSERVICE = "sling.service.subservice";

    /**
     * The logger.
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractActivityStreamProvider.class);

    /**
     * Reference to ResourceResolverFactory. Can't remove this it is API, but internally it is not used.
     */
    @Reference
    protected ResourceResolverFactory resourceResolverFactory;
    @Reference
    protected SocialActivityManager activityManager;

    @Reference
    protected final SlingRepository repository = null;

    String servicePid;

    /** The context. */
    private volatile ComponentContext context;

    /** The bound extension. */
    private final List<ActivityStreamProviderExtension> boundExtensions =
        new LinkedList<ActivityStreamProviderExtension>();

    /**
     * The list of extensions
     */
    private final CopyOnWriteArrayList<ComparableStreamProviderExtension> extensions =
        new CopyOnWriteArrayList<ComparableStreamProviderExtension>();
    /**
     * Because this is an abstract component we can't just add @Reference without bumping the package API major
     * version. So we avoid using Declarative Services and just use the next layer down directly.
     */
    private com.adobe.cq.social.serviceusers.internal.ServiceUserWrapper serviceUserWrapper;
    /**
     * Using this one lets us configure the service user with the symbolic name of *this* bundle, not the symbolic
     * name of each bundle implementing a concrete version of this component.
     */
    private ResourceResolverFactory privateResourceResolverFactory;

    public com.adobe.cq.social.serviceusers.internal.ServiceUserWrapper getServiceUserWrapper() {
		return null;
	}

    public ResourceResolverFactory getPrivateResourceResolverFactory() {
		return null;
	}

    @Override
    public void append(final Activity activity) throws ActivityException {
	}

    protected abstract ActivityStream getStream(Activity activity, ResourceResolver resolver)
        throws ActivityException {
		return null;
	}

    /**
     * Get the session to be used by the stream
     * @return the admin session or null if there is an error.
     * @throws RepositoryException
     */
    protected abstract Session getAdminSession() throws RepositoryException {
		return null;
	}

    /**
     * Activate this service. Subclass component needs to call this method in its activate method.
     * @param ctx
     */
    protected void activate(final ComponentContext ctx) throws LoginException {
	}

    /**
     * Deactivate this service. Subclass component needs to call this method in its deactivate method.
     * @param context OSGi component context
     */
    protected void deactivate(final ComponentContext context) {
	}

    protected ResourceResolver getServiceResourceResolver(final Session session) throws LoginException {
		return null;
	}

    public void bindExtension(final ActivityStreamProviderExtension extension) {
	}

    private synchronized void addExtension(final ActivityStreamProviderExtension extension) {
	}

    public void bindListenerExtension(final EventListenerExtension extension) {
	}

    public void unbindListenerExtension(final EventListenerExtension extension) {
	}

    private boolean validateExtension(final ActivityStreamProviderExtension extension) {
		return false;
	}

    private boolean hasExtension(final ActivityStreamProviderExtension extension) {
		return false;
	}

    public void unbindExtension(final ActivityStreamProviderExtension extension) {
	}

    private synchronized void unbindExtension(final String name) {
	}

    @Override
    public List<ActivityStreamProviderExtension> getExtensions() {
		return null;
	}

    /**
     * Wrapping old EventListenerExtension to support existing implementation.
     * @author thuynh
     */
    @SuppressWarnings("deprecation")
    protected class ActivityStreamProviderExtensionWrapper implements ActivityStreamProviderExtension {
        private EventListenerExtension eventListenerExtension;
        private final String[] STREAM_PROVIDER_PID = {"*"};

        public ActivityStreamProviderExtensionWrapper(final EventListenerExtension eventListenerExtension) {
            this.eventListenerExtension = eventListenerExtension;
        }

        @Override
        public String getName() {
			return null;
		}

        @Override
        public String[] getStreamProviderPid() {
			return null;
		}

        @Override
        public boolean evaluate(SocialEvent<?> event, Resource resource) {
			return false;
		}

        @Override
        public int getRanking() {
			return 0;
		}

        @Override
        public Map<String, ? extends Object> getActivityProperties(SocialEvent<?> event, Resource resource) {
			return null;
		}

        @Override
        public Map<String, ? extends Object> getObjectProperties(SocialEvent<?> event, Resource resource) {
			return null;
		}

        @Override
        public Map<String, ? extends Object> getActorProperties(SocialEvent<?> event, Resource resource) {
			return null;
		}

        @Override
        public Map<String, ? extends Object> getTargetProperties(SocialEvent<?> event, Resource resource) {
			return null;
		}
    }
}
