/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.scf;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.adobe.cq.social.scf.core.BaseQueryRequestInfo;
import com.adobe.cq.social.scf.core.CollectionSortedOrder;

/**
 * The QueryRequestInfo maintains information such as pagination, query predicates of a client request.
 * @author thuynh
 */
public interface QueryRequestInfo {

    public static String GET = "GET";
    public static List<String> SCF_SELECTORS = Arrays.asList(new String[]{"topic", "entry", "idea", "event", "asset",
        "translate"});

    public interface QueryRequestInfoFactory {
        QueryRequestInfo create() {
			return null;
		}

        QueryRequestInfo create(QueryRequestInfo copyFrom) {
			return null;
		}
    }

    QueryRequestInfoFactory DEFAULT_QUERY_INFO_FACTORY = new QueryRequestInfoFactory() {
        @Override
        public QueryRequestInfo create() {
			return null;
		}

        @Override
        public QueryRequestInfo create(final QueryRequestInfo copyFrom) {
			return null;
		}
    };

    /**
     * Is the request a query request
     */
    boolean isQuery() {
		return false;
	}

    /**
     * Is the request a translate request
     */
    boolean isTranslationRequest() {
		return false;
	}

    /**
     * Is the request a sort request
     */
    boolean isSortRequest() {
		return false;
	}

    /**
     * Sets the property that identifies if the QueryRequest is a sort request.
     * @param isSortRequest the new sort request
     */
    void setSortRequest(boolean isSortRequest) {
	}

    /**
     * Set the translation request
     * @param translation request value
     */
    void setTranslationRequest(boolean isTranslationRequest) {
	}

    /**
     * Get the query predicates.
     * @return null if the request is not a query request, otherwise, the method returns a map of the query predicates
     */
    Map<String, String[]> getPredicates() {
		return null;
	}

    /**
     * Get the request paging specification
     */
    CollectionPagination getPagination() {
		return null;
	}

    /**
     * Set the pagination
     * @param pagination the pagination setting.
     */
    void setPagination(final CollectionPagination pagination) {
	}

    /**
     * @param a list of indexes to use to sort the list by
     */
    void setSortBy(final List<String> sortBy) {
	}

    /**
     * @return the list of index names requested to sort the list by
     */
    List<String> getSortBy() {
		return null;
	}

    /**
     * Gets the sort fields. Its a ordered list. For a sort field [{"subject", true},{"added", false}] like this would
     * sort UGC based on subject in ascending order and then by create date
     * @return the sort fields
     */
    List<Entry<String, Boolean>> getSortFields() {
		return null;
	}

    /**
     * Sets the sort fields.
     * @param sortFields the sort fields
     */
    void setSortFields(final List<Entry<String, Boolean>> sortFields) {
	}

    /**
     * Gets the query params.
     * @param queryInfo the query info
     * @return the query params
     */
    String getQueryString() {
		return null;
	}

    /**
     * @param order for the list
     */
    @Deprecated
    void setSortOrder(final CollectionSortedOrder order) {
	}

    /**
     * @return the requested order to sort the list by
     */
    @Deprecated
    CollectionSortedOrder getSortOrder() {
		return null;
	}
}
