/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.scf.core;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.CheckForNull;
import javax.annotation.Nullable;
import javax.jcr.RepositoryException;
import javax.jcr.ValueFormatException;

import org.apache.commons.lang3.StringUtils;
import org.apache.jackrabbit.api.security.user.Authorizable;
import org.apache.jackrabbit.api.security.user.UserManager;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.badging.api.BadgingService;
import com.adobe.cq.social.badging.api.UserBadge;
import com.adobe.cq.social.community.api.CommunityContext;
import com.adobe.cq.social.scf.ClientUtilities;
import com.adobe.cq.social.scf.User;
import com.adobe.cq.social.scoring.api.ScoringService;
import com.adobe.cq.social.ugcbase.CollabUser;
import com.adobe.cq.social.ugcbase.SocialUtils;
import com.adobe.cq.social.ugcbase.SocialUtils.AVATAR_SIZE;
import com.adobe.cq.social.ugcbase.core.SocialResourceUtils;
import com.adobe.granite.security.user.UserProperties;
import com.adobe.granite.security.user.UserPropertiesManager;
import com.day.cq.wcm.api.Page;

public abstract class AbstractUser extends BaseSocialComponent implements User {
    /** Default avatar path. */
    static final String DEFAULT_AVATAR_PATH = SocialUtils.DEFAULT_AVATAR;
    private static final Logger LOG = LoggerFactory.getLogger(AbstractUser.class);

    private static final String PN_NUMBER_OF_POSTS = "numberOfPosts";

    private static final String FORUM_PATH = "forum";

    private static final String HTML_EXT = ".html";

    private static final String PROFILE_PATH = "profile";

    private static final String JCR_TITLE_PROP = "jcr:title";
    private static final String DISPLAY_NAME_PROP = "displayName";
    private static final String PROPERTY_LIMIT_USER_UGC = "limitUserUGC";

    private UserProperties userProps;

    private Authorizable user;

    private String name;
    private String avatarPath;
    private String userId;
    private String largeAvatarPath;
    private ScoringService scoring;
    private String profileUrl;
    private BadgingService badging;

    /**
     * Create a user based on the resource. The resource should be User.SOCIAL_AUTHORS_PREFIX + /userid.
     * @param resource the resource to try and get a User for
     * @param clientUtils client utils used for the social component
     * @param upm user properties manager for getting profile details
     * @param scoring used to determine badging in real time.
     */
    @Deprecated
    public AbstractUser(final Resource resource, final ClientUtilities clientUtils, final UserPropertiesManager upm,
        @Nullable final ScoringService scoring) {
        this(resource, clientUtils, upm);
        this.scoring = scoring;
    }

    /**
     * Create a user based on the resource. The resource should be User.SOCIAL_AUTHORS_PREFIX + /userid.
     * @param resource the resource to try and get a User for
     * @param clientUtils client utils used for the social component
     * @param upm user properties manager for getting profile details
     * @param scoring used to determine scoring in real time.
     * @param badging used to determine badging in real time.
     */
    public AbstractUser(final Resource resource, final ClientUtilities clientUtils, final UserPropertiesManager upm,
        @Nullable final ScoringService scoring, @Nullable final BadgingService badging) {
        this(resource, clientUtils, upm);
        this.scoring = scoring;
        this.badging = badging;
    }

    private boolean initialized = false;
    private final UserPropertiesManager upm;

    private void init() {
	}

    /**
     * Create a user based on the resource. The resource should be User.SOCIAL_AUTHORS_PREFIX + /userid.
     * @param resource the resource to try and get a User for
     * @param clientUtils client utils used for the social component
     * @param upm user properties manager for getting profile details
     */
    public AbstractUser(final Resource resource, final ClientUtilities clientUtils, final UserPropertiesManager upm) {
        super(resource, clientUtils);
        this.upm = upm;
        init();
    }

    private void setUpDefaults() {
	}

    @CheckForNull
    private UserProperties getUserProperties(final Resource resource, final UserPropertiesManager upm)
        throws RepositoryException {
			return null;
		}

    @Override
    public String getName() {
		return null;
	}

    @Override
    public String getAvatarUrl() {
		return null;
	}

    @Override
    public String getLargeAvatarUrl() {
		return null;
	}

    @Override
    public String getAuthorizableId() {
		return null;
	}

    @Override
    public String getPath() {
		return null;
	}

    @Override
    public String getUserId() {
		return null;
	}

    @Override
    public String getProfileUrl() {
		return null;
	}

    @Override
    public String getFriendlyUrl() {
		return null;
	}

    @Override
    @Deprecated
    public Long getNumberOfPosts() {
		return null;
	}

    @Override
    public List<Badge> getBadges() {
		return null;
	}

    @Override
    public Boolean getDisabled() {
		return null;
	}

    @Override
    public Boolean isUserUGCLimited() {
		return null;
	}

    @Override
    public String getDisabledReason() {
		return null;
	}

    /**
     * @returns the {@link UserProperties} object associated with the the user that is represented by this instance.
     */
    protected UserProperties getUserProperties() {
		return null;
	}
}
