/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.srp.internal;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import org.apache.sling.api.resource.PersistenceException;

import aQute.bnd.annotation.ProviderType;

import com.adobe.cq.social.srp.FacetRangeField;
import com.adobe.cq.social.srp.FacetSearchResult;
import com.adobe.cq.social.srp.SearchSortField;
import com.adobe.cq.social.srp.SocialResourceSearchResult;
import com.adobe.cq.social.srp.config.SocialResourceConfiguration;

/**
 * Interface that should be implemented by anybody who wants the social ugc data to be indexed for better efficient
 * searching EG: Solr, ElasticSearch, etc
 */
@ProviderType
public interface SocialIndexer {
    /**
     * To allow sharding on tenant id. Uses reportSuite from configuration
     */
    public static final String REPORT_SUITE = "report_suite";

    public void setupClient(final SocialResourceConfiguration genericConfiguration) throws IOException {
	}

    public void addToIndex(final Map<String, Object> data) throws IOException {
	}

    public void delete(String key) {
	}

    public FacetResults facetedSearch(final String queryToAs, final List<String> facetFields,
        final List<String> pivotFields, final String rows, final String mincount, final boolean isPivot,
        final SocialDatabase database) throws PersistenceException {
		return null;
	}

    public List<Map<String, Object>> getMLTResults(final String query, final String statusFilter,
        final String resourceTypeFilter, final String componentFilter, final List<String> mltField,
        final int maxResults, final int minTermFreq, final int minDocFreq, final SocialDatabase database)
        throws IOException {
		return null;
	}

    public SocialResourceSearchResult<Map<String, Object>> find(String component, String solrQueryString,
        List<SearchSortField> sortFields, int offset, int limit, final SocialDatabase database,
        Map<String, Object> signals) throws IOException {
		return null;
	}

    public FacetSearchResult findFacets(List<String> countFields, List<FacetRangeField> rangeFields,
        String resourceTypeFilter, String componentFilter, int count, boolean visibleOnly, boolean includeChildren)
        throws IOException {
		return null;
	}

    SocialResourceSearchResult<Map<String, Object>> findFacets(final List<String> inPath,
        final List<String> notInPath, final List<String> fieldNames, final List<FacetRangeField> facetRanges,
        final String scoredQueryString, final int maxFacetCount, final List<SearchSortField> sortFields,
        final int offset, final int pageSize, final SocialDatabase database) throws IOException {
		return null;
	}

    public long countChildren(String keyValue, String baseType, boolean visibleOnly) throws PersistenceException {
		return 0L;
	}

    public List<Map<String, Object>> nonFacetedSearch(String solrQueryString, final SocialDatabase database)
        throws PersistenceException {
		return null;
	}

    public void setSolrConfig(final int solrZKTimeout, final int commitWithin, AbstractSchemaMapper mapper) {
	}

    /**
     * Simply provide access to the same method in org.apache.solr.client.solrj.util.ClientUtils
     * @param query String query
     * @return String
     */
    public String escapeQueryChars(final String query) {
		return null;
	}

    /**
     * Returns a list of 'id' matching the given key and its children
     * @param key String
     * @param limit int
     * @param offset  int
     * @return List
     * @throws IOException IOException
     */
    public List<String> readForDelete(String key, int offset, int limit) throws IOException {
		return null;
	}
}
