/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2011 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.jmx.annotation;

import java.lang.annotation.Annotation;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

import javax.management.MBeanAttributeInfo;
import javax.management.MBeanConstructorInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;
import javax.management.NotCompliantMBeanException;
import javax.management.StandardMBean;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.granite.jmx.internal.Utils;

/**
 * The extension of {@link StandardMBean} that will automatically provide JMX
 * metadata through annotations.
 * 
 * @see MBeanInfo
 * @see Description
 * @see Name
 * @see Impact
 */
public class AnnotatedStandardMBean extends StandardMBean {
    private static final Logger log = LoggerFactory.getLogger(AnnotatedStandardMBean.class);

    /**
     * Make a DynamicMBean out of the object implementation, using the specified
     * mbeanInterface class.
     * 
     * @param implementation The implementation of this MBean.
     * @param mbeanInterface The Management Interface exported by this MBean's
     * implementation. If null, then this object will use standard JMX design pattern to
     * determine the management interface associated with the given implementation.
     * @param <T> Allows the compiler to check that implementation does indeed implement
     * the class described by mbeanInterface. The compiler can only check this if
     * mbeanInterface is a class literal such as MyMBean.class.
     * @throws javax.management.NotCompliantMBeanException if the mbeanInterface does not
     * follow JMX design patterns for Management Interfaces, or if the given
     * implementation does not implement the specified interface.
     * 
     * @see StandardMBean#StandardMBean(Object, Class)
     */
    public <T> AnnotatedStandardMBean(T implementation, Class<T> mbeanInterface)
            throws NotCompliantMBeanException {
        super(implementation, mbeanInterface);
    }
    
    protected AnnotatedStandardMBean(Class<?> mbeanInterface)
            throws NotCompliantMBeanException {
        super(mbeanInterface);
    }
    
    @Override
    protected String getDescription(MBeanInfo info) {
		return null;
	}

    @Override
    protected MBeanConstructorInfo[] getConstructors(
            MBeanConstructorInfo[] ctors, Object impl) {
				return null;
			}

    @Override
    protected String getParameterName(MBeanConstructorInfo ctor,
            MBeanParameterInfo param, int sequence) {
				return null;
			}

    @Override
    protected String getDescription(MBeanConstructorInfo ctor,
            MBeanParameterInfo param, int sequence) {
				return null;
			}

    @Override
    protected String getDescription(MBeanAttributeInfo info) {
		return null;
	}

    @Override
    protected String getDescription(MBeanOperationInfo info) {
		return null;
	}

    @Override
    protected int getImpact(MBeanOperationInfo info) {
		return 0;
	}

    @Override
    protected String getParameterName(MBeanOperationInfo op,
            MBeanParameterInfo param, int sequence) {
				return null;
			}

    @Override
    protected String getDescription(MBeanOperationInfo op,
            MBeanParameterInfo param, int sequence) {
				return null;
			}

    protected Constructor<?> getConstructor(MBeanConstructorInfo info)
            throws ClassNotFoundException, NoSuchMethodException {
				return null;
			}

    protected <T extends Annotation> T getAnnotation(MBeanAttributeInfo info,
            Class<T> a) throws NoSuchMethodException, ClassNotFoundException {
				return null;
			}

    protected <T extends Annotation> T getParamAnnotation(
            MBeanConstructorInfo ctor, int sequence, Class<T> clazz) {
				return null;
			}

    protected <T extends Annotation> T getParamAnnotation(
            MBeanOperationInfo op, int sequence, Class<T> clazz) {
				return null;
			}

    protected Method getReadMethod(MBeanAttributeInfo info)
            throws NoSuchMethodException {
				return null;
			}

    protected Method getWriteMethod(MBeanAttributeInfo info)
            throws NoSuchMethodException, ClassNotFoundException {
				return null;
			}

    protected Method getMethod(MBeanOperationInfo info)
            throws ClassNotFoundException, NoSuchMethodException {
				return null;
			}
}
