/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2016 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.ui.components;

import java.util.Iterator;

import org.apache.commons.collections4.Predicate;
import org.apache.commons.collections4.Transformer;
import org.apache.commons.collections4.iterators.FilterIterator;
import org.apache.commons.collections4.iterators.TransformIterator;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceWrapper;
import org.apache.sling.api.resource.ValueMap;

/**
 * A ResourceWrapper that filters its descendant resources based on <code>granite:hide</code> BooleanEL property.
 *
 * <p>
 * This wrapper needs to be applied to the container resource, such as the page root resource, and the dialog root
 * resource.
 * </p>
 *
 * <p>
 * It is also smart enough to handle include component. In that case, if the resource type is
 * <code>granite/ui/components/coral/foundation/include</code>, the included resource will also be wrapped by this
 * class, allowing the descendant resources of the included resource to use <code>granite:hide</code> property.
 * </p>
 *
 * <p>
 * For example given the following content structure:
 * </p>
 *
 * <pre>
 * <code>
 * + /mycontainer
 *   - sling:resourceType = "my/container"
 *   + item1
 *     - sling:resourceType = "my/item"
 *   + item2
 *     - sling:resourceType = "my/item"
 *     - granite:hide = true
 *   + item3
 *     - sling:resourceType = "my/item"
 *   + item4
 *     - sling:resourceType = "granite/ui/components/coral/foundation/include"
 *     - path = "/mycontainer2"
 *
 * + /mycontainer2
 *   - sling:resourceType = "my/container2"
 *   + item1
 *     - sling:resourceType = "my/item"
 *   + item2
 *     - sling:resourceType = "my/item"
 *     - granite:hide = true
 * </code>
 * </pre>
 *
 * <p>
 * Assuming that <code>my/container</code> resource type implementation uses {@code FilteringResourceWrapper} to wrap
 * its resource (<code>/mycontainer</code> resource), <code>/mycontainer/item2</code> and <code>/mycontainer2/item2</code>
 * will be excluded during traversal transparently, as if they are not there in the first place.
 * </p>
 */
public class FilteringResourceWrapper extends ResourceWrapper {
    private ExpressionResolver expressionResolver;
    private SlingHttpServletRequest request;

    public FilteringResourceWrapper(Resource resource, ExpressionResolver expressionResolver,
            SlingHttpServletRequest request) {
        super(resource);
        this.expressionResolver = expressionResolver;
        this.request = request;
    }

    @Override
    public Resource getChild(String relPath) {
		return null;
	}

    @Override
    public Iterator<Resource> listChildren() {
		return null;
	}

    @Override
    public Iterable<Resource> getChildren() {
		return null;
	}

    @Override
    public boolean hasChildren() {
		return false;
	}

    @Override
    public String getResourceType() {
		return null;
	}

    private boolean isVisible(Resource r) {
		return false;
	}
}
