/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.workflow.model;

import com.adobe.granite.workflow.HasMetaData;

import java.util.List;

/**
 * <code>WorkflowNode</code> represents a node/step in a
 * <code>{@link WorkflowModel}</code>.
 */
public interface WorkflowNode extends HasMetaData {
    public static final String TYPE_START = "START";
    public static final String TYPE_END = "END";
    public static final String TYPE_AND_SPLIT = "AND_SPLIT";
    public static final String TYPE_AND_JOIN = "AND_JOIN";
    public static final String TYPE_OR_SPLIT = "OR_SPLIT";
    public static final String TYPE_OR_JOIN = "OR_JOIN";
    public static final String TYPE_PROCESS = "PROCESS";
    public static final String TYPE_CONTAINER = "CONTAINER";
    public static final String TYPE_PARTICIPANT = "PARTICIPANT";
    public static final String TYPE_DYNAMIC_PARTICIPANT = "DYNAMIC_PARTICIPANT";
    public static final String TYPE_EXTERNAL_PROCESS = "EXTERNAL_PROCESS";

    /**
     * Returns the ID of the <code>WorkflowNode</code>.
     *
     * @return ID of the <code>WorkflowNode</code>.
     */
    String getId() {
		return null;
	}

    /**
     * Returns the type of the <code>WorkflowNode</code>. Must be unique
     * across all available types of <code>WorkflowNode</code>s.
     *
     * @return The type of the <code>WorkflowNode</code>.
     */
    String getType() {
		return null;
	}

    /**
     * Sets the type of the <code>WorkflowNode</code>. Must be unique across
     * all available types of <code>WorkflowNode</code>s.
     *
     * @param type The new type the the <code>WorkflowNode</code>.
     */
    void setType(String type) {
	}

    /**
     * Returns the description of the <code>WorkflowNode</code>.
     *
     * @return The description of the <code>WorkflowNode</code>.
     */
    String getDescription() {
		return null;
	}

    /**
     * Set the description of the <code>WorkflowNode</code>.
     *
     * @param description The new description of the <code>WorkflowNode</code>.
     */
    void setDescription(String description) {
	}

    /**
     * Returns the outgoing <code>{@link WorkflowTransition}</code>s assigned
     * to the <code>WorkflowNode</code>.
     *
     * @return <code>{@link WorkflowTransition}</code>s associated with the
     *         <code>WorkflowNode</code> (only outgoing).
     */
    List<WorkflowTransition> getTransitions() {
		return null;
	}

    /**
     * Returns the incoming <code>{@link WorkflowTransition}</code>s assigned
     * to the <code>WorkflowNode</code>.
     *
     * @return <code>{@link WorkflowTransition}</code>s associated with the
     *         <code>WorkflowNode</code> (only incoming).
     */
    List<WorkflowTransition> getIncomingTransitions() {
		return null;
	}

    /**
     * Returns the <code>WorkflowNode</code>s title.
     *
     * @return The title of the <code>WorkflowNode</code>.
     */
    String getTitle() {
		return null;
	}

    /**
     * Sets the title of the <code>WorkflowNode</code>.
     *
     * @param title The new title of the <code>WorkflowNode</code>.
     */
    void setTitle(String title) {
	}
}
