/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.commons.date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.TimeZone;

/**
 * <code>DateUtil</code> is a utility class that provides easy access to
 * commonly used dates and for parsing/reading ISO8601 date strings.
 */
public class DateUtil {

    /**
     * default logger
     */
    private static final Logger log = LoggerFactory.getLogger(DateUtil.class);

    /**
     * First day of the week. Defaults to Monday.
     */
    private final int firstDayOfWeek;
    
    /**
     * Initializes a DateUtil with the first day of week being the Monday.
     */
    public DateUtil() {
        this.firstDayOfWeek = Calendar.MONDAY;
    }

    /**
     * Initializes a DateUtil with a custom first day of the week.
     *
     * @param firstDayOfWeek the first day of the week.
     * @see Calendar#DAY_OF_WEEK
     */
    public DateUtil(int firstDayOfWeek) {
        this.firstDayOfWeek = firstDayOfWeek;
    }

    /**
     * @return calendar that represents the beginning of the last year.
     */
    public final Calendar getLastYearStart() {
		return null;
	}

    /**
     * @return calendar that represents the beginning of this year.
     */
    public final Calendar getYearStart() {
		return null;
	}

    /**
     * @return calendar that represents the date three months ago.
     */
    public final Calendar getThreeMonthsAgo() {
		return null;
	}

    /**
     * @return calendar that represents the beginning of this month.
     */
    public final Calendar getMonthStart() {
		return null;
	}

    /**
     * @return calendar that represents the start of this week. Depends on
     * {@link Calendar#setFirstDayOfWeek(int)}!
     */
    public final Calendar getWeekStart() {
		return null;
	}

    /**
     * @return calendar that represents the current date.
     */
    public final Calendar getToday() {
		return null;
	}

    /**
     * @return calendar that represents the current time.
     */
    public final Calendar getNow() {
		return null;
	}

    /**
     * Parse the given string in ISO 8601 format and build a Calendar object. If
     * no timezone is given in the string, it will use the system's default
     * timezone.
     * 
     * @param iso8601Date
     *            the date in ISO 8601 format
     * @return a Calendar instance
     * @exception InvalidDateException
     *                if the date string is not valid
     */
    public static Calendar parseISO8601(String iso8601Date) throws InvalidDateException {
		return null;
	}

    /**
     * Parse the given string in ISO 8601 format and build a Calendar object.
     * Allows to define the timezone if the string does not contain a timezone
     * reference. Otherwise the returned calendar will always have the timezone
     * that is in the ISO 8601 string.
     * 
     * @param iso8601Date
     *            the date in ISO 8601 format
     * @param defaultTimeZone
     *            the timezone to use when the timezone is not specified in the
     *            iso-8601 string (if <code>null</code>, the system default timezone will be used)
     * @return a Calendar instance
     * @exception InvalidDateException
     *                if the date string is not valid
     */
    public static Calendar parseISO8601(String iso8601Date, TimeZone defaultTimeZone) throws InvalidDateException {
		return null;
	}

    /**
     * Generate a full ISO 8601 date: "YYYY-MM-DDTHH:mm:ss.SSSZ".
     * 
     * @param calendar
     *            a Calendar instance
     * @return a string representing the date in the full ISO 8601 format:
     *         "YYYY-MM-DDTHH:mm:ss.SSSZ"
     */
    public static String getISO8601Date(Calendar calendar) {
		return null;
	}

    /**
     * Generate a full ISO 8601 date: "YYYY-MM-DDTHH:mm:ss.SSSZ". It assumes the
     * timezone of the date to be GMT (or UTC, "Z" at the end of ISO8601).
     * 
     * @param date
     *            a Date instance representing a UTC time
     * @return a string representing the date in the full ISO 8601 format:
     *         "YYYY-MM-DDTHH:mm:ss.SSSZ"
     */
    public static String getISO8601Date(Date date) {
		return null;
	}

    /**
     * Generate a full ISO 8601 date: "YYYY-MM-DDTHH:mm:ss.SSSZ". It uses the
     * given timezone for interpreting the date value.
     * 
     * @param date
     *            a Date instance
     * @param timeZone
     *            the timeZone of the date
     * 
     * @return a string representing the date in the full ISO 8601 format:
     *         "YYYY-MM-DDTHH:mm:ss.SSSZ"
     */
    public static String getISO8601Date(Date date, TimeZone timeZone) {
		return null;
	}
    
    /**
     * Generate a ISO 8601 date with date and time, but without the milliseconds
     * part: "YYYY-MM-DDTHH:mm:ssZ".
     * 
     * @param calendar
     *            a Calendar instance
     * @return a string representing date and time except the milliseconds in
     *         the ISO 8601 format: "YYYY-MM-DDTHH:mm:ssZ"
     */
    public static String getISO8601DateAndTimeNoMillis(Calendar calendar) {
		return null;
	}

    /**
     * Generate a ISO 8601 date with date and time, but without the milliseconds
     * part: "YYYY-MM-DDTHH:mm:ssZ". It assumes the timezone of the date to be
     * GMT (or UTC, "Z" at the end of ISO8601).
     * 
     * @param date
     *            a Date instance representing a UTC time
     * @return a string representing date and time except the milliseconds in
     *         the ISO 8601 format: "YYYY-MM-DDTHH:mm:ssZ"
     */
    public static String getISO8601DateAndTimeNoMillis(Date date) {
		return null;
	}

    /**
     * Generate a ISO 8601 date with date and time, but without the milliseconds
     * part: "YYYY-MM-DDTHH:mm:ssZ". It uses the given timezone for interpreting
     * the date value.
     * 
     * @param date
     *            a Date instance
     * @param timeZone
     *            the timeZone of the date
     * @return a string representing date and time except the milliseconds in
     *         the ISO 8601 format: "YYYY-MM-DDTHH:mm:ssZ"
     */
    public static String getISO8601DateAndTimeNoMillis(Date date, TimeZone timeZone) {
		return null;
	}

    /**
     * Generate a ISO 8601 date in the pure date form: "YYYY-MM-DD"
     * 
     * @param calendar
     *            a Calendar instance
     * @return a string representing only the date in the ISO 8601 format:
     *         "YYYY-MM-DD"
     */
    public static String getISO8601DateNoTime(Calendar calendar) {
		return null;
	}

    /**
     * Generate a ISO 8601 date in the pure date form: "YYYY-MM-DD". It assumes
     * the timezone of the date to be GMT (or UTC, "Z" at the end of ISO8601).
     * 
     * @param date
     *            a Date instance representing a UTC time.
     * @return a string representing only the date in the ISO 8601 format:
     *         "YYYY-MM-DD"
     */
    public static String getISO8601DateNoTime(Date date) {
		return null;
	}

    /**
     * Generate a ISO 8601 date in the pure date form: "YYYY-MM-DD". It uses the
     * given timezone for interpreting the date value.
     * 
     * @param date
     *            a Date instance representing a date
     * @param timeZone
     *            the timeZone of the date
     * @return a string representing only the date in the ISO 8601 format:
     *         "YYYY-MM-DD"
     */
    public static String getISO8601DateNoTime(Date date, TimeZone timeZone) {
		return null;
	}

    private static String twoDigit(int i) {
		return null;
	}

    private static String threeDigit(int i) {
		return null;
	}

    private static boolean check(StringTokenizer st, String token)
            throws InvalidDateException {
				return false;
			}

    /**
     * Returns a date format object, based on a date pattern and a <code>Locale</code>
     *
     * @param pattern Date pattern
     * @param locale Date format localisation
     * @return Date format or <code>null</code>
     */
    public static DateFormat getDateFormat(String pattern, Locale locale) {
		return null;
	}

    /**
     * Returns a date format object, based on either a custom date pattern or a default date pattern, and a <code>Locale</code>
     *
     * @param pattern        User provided date pattern
     * @param defaultPattern Default date format used if the user provided date pattern was not valid
     * @param locale Date format localisation
     * @return Custom date format (if valid) or default date format
     */
    public static DateFormat getDateFormat(String pattern, String defaultPattern, Locale locale) {
		return null;
	}

    /**
     * Returns a date format object, based on either a custom date pattern or a default date pattern, and a <code>Locale</code>
     *
     * @param pattern        User provided date pattern
     * @param defaultDateFormat Default date format used if the user provided date pattern was not valid
     * @param locale Date format localisation
     * @return Custom date format (if valid) or default date format
     */
    public static DateFormat getDateFormat(String pattern, DateFormat defaultDateFormat, Locale locale) {
		return null;
	}
}
