/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.security.util;

import java.io.Writer;
import java.security.Principal;
import java.util.Calendar;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;

import com.adobe.granite.security.user.UserProperties;
import com.adobe.granite.security.user.UserPropertiesManager;
import com.adobe.granite.security.user.UserPropertiesService;
import com.day.cq.commons.JSONWriterUtil;
import com.day.cq.replication.ReplicationQueue;
import com.day.cq.replication.ReplicationStatus;
import com.day.cq.security.profile.ProfileManager;
import com.day.cq.xss.XSSProtectionService;
import org.apache.jackrabbit.api.JackrabbitSession;
import org.apache.jackrabbit.api.security.principal.PrincipalIterator;
import org.apache.jackrabbit.api.security.user.Authorizable;
import org.apache.jackrabbit.api.security.user.Group;
import org.apache.jackrabbit.api.security.user.User;
import org.apache.jackrabbit.api.security.user.UserManager;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.commons.json.JSONException;
import org.apache.sling.commons.json.io.JSONWriter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static com.day.cq.commons.jcr.JcrConstants.JCR_CONTENT;
import static com.day.cq.commons.jcr.JcrConstants.JCR_CREATED;
import static com.day.cq.commons.jcr.JcrConstants.JCR_CREATED_BY;
import static com.day.cq.commons.jcr.JcrConstants.JCR_LASTMODIFIED;

/**
 * Little writer knowing some of the Authorizables special cases that do not
 * compile a Bean-convention
 *
 * @deprecated cq 5.5 Use platform functionality instead.
 */
public class AuthorizableJSONWriter {

    /**
     * Maps user property names to profile properties. This is only needed
     * for backward compatibility reasons. See bug# 26797.
     */
    private static final Map<String, String> USER_TO_PROFILE_PROPS;

    static {
        Map<String, String> tmp = new HashMap<String, String>();
        tmp.put(com.day.cq.security.Authorizable.PROPERTY_DESCRIPTION, UserProperties.ABOUT_ME);
        tmp.put(com.day.cq.security.Authorizable.PROPERTY_EMAIL, UserProperties.EMAIL);
        tmp.put(com.day.cq.security.Authorizable.PROPERTY_FIRST_NAME, UserProperties.GIVEN_NAME);
        tmp.put(com.day.cq.security.Authorizable.PROPERTY_LAST_NAME, UserProperties.FAMILY_NAME);
        USER_TO_PROFILE_PROPS = tmp;
    }

    private final static Logger log = LoggerFactory.getLogger(AuthorizableJSONWriter.class);

    public static final String GROUP = "group";

    public static final String USER = "user";

    public static final String KEY_HOME_PATH = "home";

    private final ResourceResolver resourceResolver;
    private final Session session;

    private final UserPropertiesManager pMgr;

    private Set<String> outputProps;

    private XSSProtectionService xss;

    private boolean basic;

    private int membersLimit = -1;

    /**
     * Creates a default authorizable writer.
     */
    public AuthorizableJSONWriter() {
        throw new UnsupportedOperationException("No longer supported (AuthorizableJSONWriter())");
    }

    /**
     * todo
     * @param pMgr the profile manager
     * @param session the session
     * @param outputProps the output properties
     * @param xss the XSS protection service
     * @since 5.4
     */
    public AuthorizableJSONWriter(ProfileManager pMgr, Session session, String[] outputProps, XSSProtectionService xss) {
        throw new UnsupportedOperationException("No longer supported (AuthorizableJSONWriter(ProfileManager, Sesion, String[], XSSProtectionService))");
    }

    public AuthorizableJSONWriter(ResourceResolver resourceResolver, UserPropertiesManager pMgr, String[] outputProps, XSSProtectionService xss) {
        this.resourceResolver = resourceResolver;
        this.session = (resourceResolver == null) ? null : resourceResolver.adaptTo(Session.class);
        this.pMgr = pMgr;
        setOutputProps(outputProps);
        this.xss = xss;
    }

    /**
     * Sets the members limit.
     * @param membersLimit the limit
     */
    public void setMembersLimit(int membersLimit) {
	}

    /**
     * Set the properties that will be present in the output.
     * @param outputProps the output properties
     * @since 5.4
     */
    public void setOutputProps(String[] outputProps) {
	}

    /**
     * Define if only basic information should be written
     * @param basic <code>true</code> to only dump basic information
     */
    public void setBasic(boolean basic) {
	}

    /**
     * Set XSS protection service to use to prevent from XSS attacks
     *
     * @param xss
     * @since 5.4
     */
    public void setXss(XSSProtectionService xss) {
	}

    public void write(JSONWriter writer, com.day.cq.security.Authorizable authorizable) throws JSONException {
	}

    public void write(JSONWriter writer, Authorizable authorizable) throws JSONException, RepositoryException {
	}

    public void write(Writer out, com.day.cq.security.Authorizable authorizable) throws JSONException {
	}

    public void write(Writer out, Authorizable authorizable) throws JSONException, RepositoryException {
	}

    public void writeTable(Writer out, com.day.cq.security.Authorizable authorizable) throws JSONException, RepositoryException {
	}

    public void writeTable(Writer out, Authorizable authorizable) throws JSONException, RepositoryException {
	}

    private void write(JSONWriter writer, String key, String value) throws JSONException {
	}

    private void write(JSONWriter writer, String key, long value) throws JSONException {
	}

    private void writeRows(JSONWriter json, Authorizable authorizable) throws JSONException, RepositoryException {
	}

    private static void writeColumnHeader(JSONWriter json, Authorizable authorizable) throws JSONException {
	}

    private void writeAuthorizables(JSONWriter writer, String key, Iterator<? extends Authorizable> auths, int limit, JSONWriterUtil.WriteMode writeMode)
            throws JSONException, RepositoryException {
			}

    private void writeModificationDates(JSONWriter writer, Authorizable authorizable) throws JSONException, RepositoryException {
	}

    private void writeReplication(JSONWriter writer, Authorizable authorizable) throws JSONException, RepositoryException {
	}

    private UserProperties getProfile(Authorizable authorizable) throws RepositoryException {
		return null;
	}

    private String getFormattedName(String id) {
		return null;
	}

    /**
     * Backwards compatible with CQ Authorizable#getName
     * @param authorizable
     * @return
     */
    private static String getName(Authorizable authorizable, UserProperties profile) throws RepositoryException {
		return null;
	}

    private boolean includeKey(String key) {
		return false;
	}
}
