/*
 * Copyright 1997-2009 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.wcm.api;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.osgi.service.event.Event;

/**
 * This class encapsulates the information coming along with a page
 * modification.
 */
public final class PageModification implements Serializable {

    private static final String PROPERTY_CHANGES = "changes";

    private static final String PROPERTY_VERSION_ID = "versionId";

    private static final String PROPERTY_USER_ID = "userId";

    private static final String PROPERTY_PATH = "path";

    private static final String PROPERTY_DESTINATION = "destination";

    private static final String PROPERTY_ABOVE = "above";

    private static final String PROPERTY_MODIFIED_DATE = "modifiedDate";

    private static final String PROPERTY_TYPE = "type";

    /**
     * The modification type.
     */
    public static enum ModificationType {

        /**
         * page was created
         */
        CREATED("PageCreated"),

        /**
         * page was modified
         */
        MODIFIED("PageModified"),

        /**
         * page was moved
         */
        MOVED("PageMoved"),

        /**
         * page was deleted
         */
        DELETED("PageDeleted"),

        /**
         * page was versioned
         */
        VERSION_CREATED("VersionCreated"),

        /**
         * page was restored
         */
        RESTORED("PageRestored"),

        /**
         * page rolled out
         */
        ROLLEDOUT("PageRolledOut"),

        /**
         * page became valid (on time reached)
         */
        VALID("PageValid"),

        /**
         * page became invalid (onff time reached)
         */
        INVALID("PageInvalid");


        private final String name;

        ModificationType(String name) {
            this.name = name;
        }

        /**
         * @see java.lang.Enum#toString()
         */
        public String toString() {
            return name;
        }

        public static ModificationType fromName(String n) {
            for (ModificationType m : ModificationType.values()) {
                if (m.toString().equals(n)) {
                    return m;
                }
            }
            throw new IllegalArgumentException("Unknown modification type: " + n);
        }
    }

    /**
     * Path of page that changed
     */
    private final String path;

    /**
     * Destination hande for move, may be <code>null</code> if this was a
     * simple reordering inside the same hierarchy
     */
    private final String destination;

    /**
     * above path (used in create/move messages)
     */
    private final String above;

    /**
     * Modification type.
     */
    private final ModificationType type;

    /**
     * The version of the page that this modification belongs to.
     */
    private final String versionId;

    /**
     * The user object of this page modification.
     */
    private final String userId;

    /**
     * The modification date.
     */
    private final Date modificationDate;

    /**
     * A list of changes.
     */
    private final Set<String> changes;

    /**
     * Create a new <code>PageModification</code> that represents
     * a page creation.
     *
     * @param path     path to new page
     * @param above    label of successor sibling
     * @param userId   the id of the user that caused this modification.
     * @return modification
     */
    public static PageModification created(String path, String above, String userId) {
		return null;
	}

    /**
     * Create a new <code>PageModification</code> that represents
     * a page modification.
     *
     * @param path    path of page
     * @param vid     the version name
     * @param userId  the id of the user that caused this modification.
     * @param changes A set of changes (paths) - optional
     * @return modification
     */
    public static PageModification modified(String path,
                                            String vid,
                                            String userId,
                                            Set<String> changes) {
												return null;
											}

    /**
     * Create a new <code>PageModification</code> that represents
     * a page moval.
     *
     * @param path        path of page
     * @param destination new page destination, may be <code>null</code>
     * @param above       label of successor sibling, may be <code>null</code>
     * @param userId      the id of the user that caused this modification.
     * @return modification
     */
    public static PageModification moved(String path,
                                         String destination, String above, String userId) {
											return null;
										}

    /**
     * Create a new <code>PageModification</code> that represents
     * a page deletion.
     *
     * @param path   path of page
     * @param userId the id of the user that caused this modification.
     * @return modification
     */
    public static PageModification deleted(String path,
                                           String userId) {
											return null;
										}

    /**
     * Create a new <code>PageModification</code> that represents
     * a version created.
     *
     * @param path   path of page
     * @param vid    the versio name
     * @param userId the id of the user that caused this modification.
     * @return modification
     */
    public static PageModification versionCreated(String path,
                                                  String vid, String userId) {
													return null;
												}

    /**
     * Create a new <code>PageModification</code> that represents
     * a version created.
     *
     * @param path   path of page
     * @param vid    the vesion name
     * @param userId the id of the user that caused this modification.
     * @return modification
     */
    public static PageModification pageRestored(String path,
                                                String vid, String userId) {
													return null;
												}

    /**
     * Create a new <code>PageModification</code> that represents
     * that the page became valid due to an on-time reached.
     *
     * @param path path of page
     * @param time the on-time
     * @return modification
     */
    public static PageModification pageValid(String path, long time) {
		return null;
	}

    /**
     * Create a new <code>PageModification</code> that represents
     * that the page became invalid due to an off-time reached.
     *
     * @param path path of page
     * @param time the off-time
     * @return modification
     */
    public static PageModification pageInvalid(String path, long time) {
		return null;
	}

    /**
     * Create a new <code>PageModification</code> that represents
     * a version deleted.
     *
     * @param path   path of page
     * @param userId the id of the user that caused this modification.
     * @return modification
     */
    public static PageModification rolledout(String path,
                                             String userId) {
												return null;
											}

    /**
     * Create a new <code>PageModification</code>. Internal method called
     * by one of the static factory methods {@link #created},
     * {@link #modified}, {@link #moved}, {@link #deleted} or
     * {@link #versionCreated}
     *
     * @param type        modification type
     * @param path        path of the page; must not be <code>null</code>
     * @param versionid   The versionid for the modification
     * @param destination destination in case of move operations, otherwise
     *                    <code>null</code>
     * @param above       path above in case of create/move operations, otherwise
     *                    <code>null</code>
     * @param userId      the id of the user that caused this modification.
     * @param md          the modification date. if <code>null</code> the current date is used
     * @param changes     A set of changes (paths) - optional
     */
    private PageModification(ModificationType type,
                             String path,
                             String versionid,
                             String destination,
                             String above,
                             String userId,
                             Date md,
                             Set<String> changes) {
        this.type = type;
        this.path = path;
        this.versionId = versionid;
        this.destination = destination;
        this.above = above;
        this.userId = userId == null ? "admin" : userId;
        this.modificationDate = (md == null ? new Date() : md);
        this.changes = changes;
    }

    /**
     * The path of the page.
     *
     * @return The path of the page.
     */
    public String getPath() {
		return null;
	}

    public String getDestination() {
		return null;
	}

    public String getAbove() {
		return null;
	}

    public ModificationType getType() {
		return null;
	}

    public String getVersionId() {
		return null;
	}

    public String getUserId() {
		return null;
	}

    public Date getModificationDate() {
		return null;
	}

    /**
     * Get the list of changes for a modification of the page.
     *
     * @return A set of relative paths or null.
     */
    public Set<String> getModificationPaths() {
		return null;
	}

    /**
     * Overridden to alter the preconditions when two modifications are
     * considered equal.
     *
     * @param o object to compare this object against
     * @return <code>true</code> if this object is considered equal to the
     *         other object, otherwise <code>false</code>
     */
    public final boolean equals(Object o) {
		return false;
	}

    /**
     * Returns a hash code value for the object. Replaced in order to
     * return the same hashcode for two objects that are considered
     * equal according to the {@link #equals} method implemented above.
     *
     * @return a hash code value for this object.
     * @see java.lang.Object#equals(java.lang.Object)
     * @see java.util.Hashtable
     */
    public int hashCode() {
		return 0;
	}

    /**
     * Return a string representation of this object
     *
     * @return string representation
     */
    public String toString() {
		return null;
	}

    /**
     * Get a dictionary with all event properties.
     *
     * @return an map of event properties
     */
    public Map<String, Object> getEventProperties() {
		return null;
	}

    /**
     * Create a page modification from properties.
     *
     * @param props the event properties
     * @return a page modification object
     */
    @SuppressWarnings("unchecked")
    public static PageModification fromEventProperties(Map<String, Object> props) {
		return null;
	}

    /**
     * Convert a single modification to an event.
     *
     * @return a page event
     */
    public Event toEvent() {
		return null;
	}
}
