/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.wcm.api.components;

import java.util.LinkedList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Collections;
import java.util.Arrays;

import org.apache.sling.commons.json.JSONException;
import org.apache.sling.commons.json.JSONObject;
import org.apache.sling.commons.json.io.JSONWriter;

/**
 * <code>Toolbar</code>...
 */
public class Toolbar implements List<Toolbar.Item> {

    private final List<Item> items;

    /**
     * Creates an empty toolbar
     */
    public Toolbar() {
         items = new LinkedList<Item>();
    }

    /**
     * Creates a new toolbar and adds the given items to it.
     * @param items items to add
     */
    public Toolbar(Item ... items) {
        this.items = new LinkedList<Item>();
        this.items.addAll(Arrays.asList(items));
    }

    /**
     * Creates a new toolbar and adds the given items to it.
     * @param items items to add
     */
    public Toolbar(Collection<? extends Item> items) {
        this.items = new LinkedList<Item>();
        this.items.addAll(items);
    }

    /**
     * Creates an unmodifiable copy of the given toolbar.
     * @param toolbar base toolbar
     */
    public Toolbar(Toolbar toolbar) {
        items = toolbar == null
                ? Collections.<Item>emptyList()
                : Collections.unmodifiableList(toolbar);
    }

    public void write(JSONWriter writer, String key) throws JSONException {
	}

    public interface Item {

        /**
         * Serialize to JSON writer.
         * @param writer the writer
         * @throws JSONException if a JSON error occurs.
         */
        void write(JSONWriter writer) throws JSONException {
		}

    }

    public static class Custom extends JSONObject implements Item {

        public void write(JSONWriter writer) throws JSONException {
		}
    }

    /**
     * Represents a toolbar button
     * @since 5.3
     */
    public static class Button extends JSONObject implements Item {

        public Button(String text, String handler) {
            this(text, handler, false, null);
        }

        public Button(String text, String handler, boolean disabled, String tooltip) {
            try {
                put("xtype", "button");
                put("text", text);
                put("handler", handler);
                if (tooltip != null) {
                    put("tooltip", tooltip);
                }
                if (disabled) {
                    put("disabled", true);
                }
            } catch (JSONException e) {
                // ignore
            }
        }

        public Button setText(String text) {
			return null;
		}

        public Button setHandler(String handler) {
			return null;
		}

        public Button setTooltip(String tooltip) {
			return null;
		}

        public Button setDisabled(boolean disabled) {
			return null;
		}

        public void write(JSONWriter writer) throws JSONException {
		}
    }


    public static class Label implements Item {

        private final String text;

        public Label(String text) {
            this.text = text;
        }

        public void write(JSONWriter writer) throws JSONException {
		}

    }

    public static class Separator implements Item {

        public void write(JSONWriter writer) throws JSONException {
		}
    }
    
    public int size() {
		return 0;
	}

    public boolean isEmpty() {
		return false;
	}

    public boolean contains(Object o) {
		return false;
	}

    public Iterator<Item> iterator() {
		return null;
	}

    public Object[] toArray() {
		return null;
	}

    public <T> T[] toArray(T[] a) {
		return null;
	}

    public boolean add(Item o) {
		return false;
	}

    public boolean remove(Object o) {
		return false;
	}

    public boolean containsAll(Collection<?> c) {
		return false;
	}

    public boolean addAll(Collection<? extends Item> c) {
		return false;
	}

    public boolean addAll(int index, Collection<? extends Item> c) {
		return false;
	}

    public boolean removeAll(Collection<?> c) {
		return false;
	}

    public boolean retainAll(Collection<?> c) {
		return false;
	}

    public void clear() {
	}

    public boolean equals(Object o) {
		return false;
	}

    public int hashCode() {
		return 0;
	}

    public Item get(int index) {
		return null;
	}

    public Item set(int index, Item element) {
		return null;
	}

    public void add(int index, Item element) {
	}

    public Item remove(int index) {
		return null;
	}

    public int indexOf(Object o) {
		return 0;
	}

    public int lastIndexOf(Object o) {
		return 0;
	}

    public ListIterator<Item> listIterator() {
		return null;
	}

    public ListIterator<Item> listIterator(int index) {
		return null;
	}

    public List<Item> subList(int fromIndex, int toIndex) {
		return null;
	}
}