/*
 * Copyright 1997-2010 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.wcm.notification.email;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.jcr.Node;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.SimpleCredentials;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.mail.Email;
import org.apache.commons.mail.EmailException;
import org.apache.commons.mail.SimpleEmail;
import org.apache.sling.api.resource.LoginException;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceResolverFactory;
import org.apache.sling.jcr.api.SlingRepository;
import org.osgi.service.event.Event;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.granite.security.user.UserProperties;
import com.adobe.granite.security.user.UserPropertiesManager;
import com.adobe.granite.security.user.UserPropertiesService;
import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.wcm.notification.NotificationContext;

public abstract class AbstractEmailBuilder implements EmailBuilder {

    protected static final String TEMPLATE_ROOT_PATH = "/etc/notification/email/default";
    protected static final String DEFAULT_CONTENT_TYPE = "text/plain; charset=utf-8";
    protected static final String DEFAULT_CHARSET = "UTF-8";
    protected final static String NOTIFICATION_SERVICE = "notification-service";


    protected abstract ResourceResolverFactory getResolverFactory() {
		return null;
	}

    private final Logger log = LoggerFactory.getLogger(this.getClass().getName());

    // -----------< Abstract Methods >-------------

    /**
     * Returns a map of additional email headers to be added to the {@link Email}.
     *
     * @param context The notification context.
     * @param event   The notification event.
     *
     * @return A {@link Map} containing additional email headers.
     */
    protected abstract Map<String, String> getHeaders(NotificationContext context, Event event) {
		return null;
	}

    /**
     * Returns the subject for the email.
     *
     * @param context    The notification context.
     * @param event      The event.
     * @param properties The text template properties loaded from the repository.
     *
     * @return The subject.
     */
    protected abstract String getSubject(NotificationContext context, Event event, Properties properties) {
		return null;
	}

    /**
     * Returns the message body of the notification email.
     *
     * @param context    The notification context.
     * @param event      The notification event.
     * @param properties The text template properties loaded from the repository.
     *
     * @return The email body / message.
     */
    protected abstract String getMessage(NotificationContext context, Event event, Properties properties) {
		return null;
	}

    /**
     * Return the content type.
     * @return the content type.
     */
    protected String getContentType() {
		return null;
	}

    /**
     * Return the charset.
     * @return the charset.
     */
    protected String getCharset() {
		return null;
	}

    /**
     * Return the template root path.
     * @return the template root path.
     */
    protected String getTemplateRootPath() {
		return null;
	}
    
    /**
     * {@inheritDoc}
     */
    public boolean shouldBuild(final NotificationContext context, final Event event) {
		return false;
	}
    
    public abstract Repository getRepository() {
		return null;
	}
    
    /**
     * {@inheritDoc}
     */
    public Email build(final NotificationContext context, final Event event, final String addressFrom) {
		return null;
	}

    /**
     * Loads the text properties for the email from a file node in the repository.
     *
     * @param context The notification context.
     * @param event   The event.
     *
     * @return A {@link Properties} object with the email text.
     *
     * @throws RepositoryException If loading the file node fails.
     * @throws IOException         If loading the input stream into the properties object fails.
     */
    protected Properties loadEmailProperties(final NotificationContext context, final Event event)
            throws RepositoryException, IOException {
				return null;
			}
}

