/*
 * Copyright 2007 ZXing authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.zxing.common.reedsolomon;

/**
 * <p>This class contains utility methods for performing mathematical operations over
 * the Galois Fields. Operations use a given primitive polynomial in calculations.</p>
 *
 * <p>Throughout this package, elements of the GF are represented as an {@code int}
 * for convenience and speed (but at the cost of memory).
 * </p>
 *
 * @author Sean Owen
 * @author David Olivier
 */
public final class GenericGF {

  public static final GenericGF AZTEC_DATA_12 = new GenericGF(0x1069, 4096); // x^12 + x^6 + x^5 + x^3 + 1
  public static final GenericGF AZTEC_DATA_10 = new GenericGF(0x409, 1024); // x^10 + x^3 + 1
  public static final GenericGF AZTEC_DATA_6 = new GenericGF(0x43, 64); // x^6 + x + 1
  public static final GenericGF AZTEC_PARAM = new GenericGF(0x13, 16); // x^4 + x + 1
  public static final GenericGF QR_CODE_FIELD_256 = new GenericGF(0x011D, 256); // x^8 + x^4 + x^3 + x^2 + 1
  public static final GenericGF DATA_MATRIX_FIELD_256 = new GenericGF(0x012D, 256); // x^8 + x^5 + x^3 + x^2 + 1
  public static final GenericGF AZTEC_DATA_8 = DATA_MATRIX_FIELD_256;
  public static final GenericGF MAXICODE_FIELD_64 = AZTEC_DATA_6;

  private static final int INITIALIZATION_THRESHOLD = 0;

  private int[] expTable;
  private int[] logTable;
  private GenericGFPoly zero;
  private GenericGFPoly one;
  private final int size;
  private final int primitive;
  private boolean initialized = false;

  /**
   * Create a representation of GF(size) using the given primitive polynomial.
   *
   * @param primitive irreducible polynomial whose coefficients are represented by
   *  the bits of an int, where the least-significant bit represents the constant
   *  coefficient
   */
  public GenericGF(int primitive, int size) {
  	this.primitive = primitive;
    this.size = size;
    
    if (size <= INITIALIZATION_THRESHOLD){
    	initialize();
    }
  }

  private void initialize(){
}
  
  private void checkInit(){
}
  
  GenericGFPoly getZero() {
	return null;
}

  GenericGFPoly getOne() {
	return null;
}

  /**
   * @return the monomial representing coefficient * x^degree
   */
  GenericGFPoly buildMonomial(int degree, int coefficient) {
	return null;
}

  /**
   * Implements both addition and subtraction -- they are the same in GF(size).
   *
   * @return sum/difference of a and b
   */
  static int addOrSubtract(int a, int b) {
	return 0;
}

  /**
   * @return 2 to the power of a in GF(size)
   */
  int exp(int a) {
	return 0;
}

  /**
   * @return base 2 log of a in GF(size)
   */
  int log(int a) {
	return 0;
}

  /**
   * @return multiplicative inverse of a
   */
  int inverse(int a) {
	return 0;
}

  /**
   * @param a
   * @param b
   * @return product of a and b in GF(size)
   */
  int multiply(int a, int b) {
	return 0;
}

  public int getSize(){
	return 0;
}
  
}
