/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.commons.json;

import java.io.BufferedReader;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.util.Stack;

/**
 * <code>JsonParser</code> parses and validates the JSON object passed upon 
 * {@link #parse(String)} or {@link #parse(InputStream, String)} and notifies
 * the specified <code>JsonHandler</code>
 */
public class JsonParser {

    private static final String NULL = "null";
    private static final int EOF = -1;

    private static final int KEY_START = 1;
    private static final int VALUE_START = 2;
    private static final int VALUE = 4;

    private static final Integer OBJECT = new Integer(8);
    private static final Integer ARRAY = new Integer(32);

    /* the handler */
    private final JsonHandler handler;

    /**
     * Create a new <code>JSONParser</code> with the specified <code>JSONHandler</code>.
     *
     * @param jsonHandler A <code>JSONHandler</code>
     */
    public JsonParser(JsonHandler jsonHandler) {
        handler = jsonHandler;
    }

    /**
     *
     * @param str String to be parsed
     * @throws IOException If an error occurs.
     */
    public void parse(String str) throws IOException {
	}

    /**
     *
     * @param input InputStream to be parsed.
     * @param charSetName Name of the charset to be used.
     * @throws IOException If an error occurs.
     */
    public void parse(InputStream input, String charSetName) throws IOException {
	}

    /**
     *
     * @param reader The reader
     * @throws IOException If an error occurs.
     */
    public void parse(Reader reader) throws IOException {
	}

    /**
     * Return the characters up to the next close quote character.
     * Backslash processing is done. The formal JSON format does not
     * allow strings in single quotes, but an implementation is allowed to
     * accept them.
     *
     * @param r The reader.
     * @param quote The quoting character, either
     *      <code>"</code>&nbsp;<small>(double quote)</small> or
     *      <code>'</code>&nbsp;<small>(single quote)</small>.
     * @return      A String.
     * @throws IOException Unterminated string.
     */
    private static String nextString(Reader r, char quote) throws IOException {
		return null;
	}

    private static String next(Reader r, int n) throws IOException {
		return null;
	}

    /**
     * Get the next char in the string, skipping whitespace.
     *
     * @param reader The reader
     * @return A character, or -1 if there are no more characters.
     * @throws IOException If an error occurs.
     */
    private static int readIgnoreWhitespace(Reader reader) throws IOException {
		return 0;
	}

    private StringBuffer resetValue(StringBuffer value) throws IOException {
		return null;
	}
}