/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.commons.query.sql2;

import java.util.BitSet;
import java.util.Arrays;

import javax.jcr.query.qom.QueryObjectModel;
import javax.jcr.query.qom.Join;
import javax.jcr.query.qom.Source;
import javax.jcr.query.qom.Selector;
import javax.jcr.query.qom.Column;
import javax.jcr.query.qom.Constraint;
import javax.jcr.query.qom.Ordering;
import javax.jcr.query.qom.JoinCondition;
import javax.jcr.query.qom.EquiJoinCondition;
import javax.jcr.query.qom.ChildNodeJoinCondition;
import javax.jcr.query.qom.DescendantNodeJoinCondition;
import javax.jcr.query.qom.SameNodeJoinCondition;
import javax.jcr.query.qom.And;
import javax.jcr.query.qom.ChildNode;
import javax.jcr.query.qom.Comparison;
import javax.jcr.query.qom.DescendantNode;
import javax.jcr.query.qom.FullTextSearch;
import javax.jcr.query.qom.Not;
import javax.jcr.query.qom.Or;
import javax.jcr.query.qom.PropertyExistence;
import javax.jcr.query.qom.SameNode;
import javax.jcr.query.qom.DynamicOperand;
import javax.jcr.query.qom.StaticOperand;
import javax.jcr.query.qom.BindVariableValue;
import javax.jcr.query.qom.Literal;
import javax.jcr.query.qom.FullTextSearchScore;
import javax.jcr.query.qom.Length;
import javax.jcr.query.qom.LowerCase;
import javax.jcr.query.qom.NodeLocalName;
import javax.jcr.query.qom.NodeName;
import javax.jcr.query.qom.PropertyValue;
import javax.jcr.query.qom.UpperCase;
import javax.jcr.query.qom.QueryObjectModelConstants;
import javax.jcr.Value;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;

/**
 * <code>QOMFormatter</code> implements a formatter that translates a query
 * object model into a JCR_SQL2 string statement.
 */
public class QOMFormatter implements QueryObjectModelConstants {

    /**
     * BitSet of valid SQL identifier start characters.
     */
    private static final BitSet IDENTIFIER_START = new BitSet();

    /**
     * BitSet of valid SQL identifier body characters.
     */
    private static final BitSet IDENTIFIER_PART_OR_UNDERSCORE = new BitSet();

    static {
        for (char c = 'a'; c <= 'z'; c++) {
            IDENTIFIER_START.set(c);
        }
        for (char c = 'A'; c <= 'Z'; c++) {
            IDENTIFIER_START.set(c);
        }
        IDENTIFIER_PART_OR_UNDERSCORE.or(IDENTIFIER_START);
        for (char c = '0'; c <= '9'; c++) {
            IDENTIFIER_PART_OR_UNDERSCORE.set(c);
        }
        IDENTIFIER_PART_OR_UNDERSCORE.set('_');
    }

    /**
     * The query object model to format.
     */
    private final QueryObjectModel qom;

    /**
     * The JCR_SQL2 statement.
     */
    private final StringBuilder sb = new StringBuilder();

    /**
     * Private constructor.
     *
     * @param qom the query object model to format.
     */
    private QOMFormatter(QueryObjectModel qom) {
        this.qom = qom;
    }

    /**
     * Formats the given <code>qom</code> as a JCR_SQL2 query statement.
     *
     * @param qom the query object model to translate.
     * @return the JCR_SQL2 statement.
     * @throws RepositoryException if an error occurs while formatting the qom.
     */
    public static String format(QueryObjectModel qom)
            throws RepositoryException {
				return null;
			}

    private String format() throws RepositoryException {
		return null;
	}

    private void append(Ordering[] orderings) {
	}

    private void append(Constraint c)
            throws RepositoryException {
			}

    private void append(And constraint)
            throws RepositoryException {
			}

    private void append(ChildNode constraint) {
	}

    private void append(Comparison constraint)
            throws RepositoryException {
			}

    private void append(StaticOperand operand)
            throws RepositoryException {
			}

    private void append(BindVariableValue value) {
	}

    private void append(Literal value)
            throws RepositoryException {
			}

    private void appendCastLiteral(String value, String propertyType) {
	}

    private void appendStringLiteral(String value) {
	}

    private void appendOperator(String operator) {
	}

    private void append(DynamicOperand operand) {
	}

    private void append(FullTextSearchScore operand) {
	}

    private void append(Length operand) {
	}

    private void append(LowerCase operand) {
	}

    private void append(NodeLocalName operand) {
	}

    private void append(NodeName operand) {
	}

    private void append(PropertyValue operand) {
	}

    private void append(UpperCase operand) {
	}

    private void append(DescendantNode constraint) {
	}

    private void append(FullTextSearch constraint) throws RepositoryException {
	}

    private void append(Not constraint) throws RepositoryException {
	}

    private void append(Or constraint) throws RepositoryException {
	}

    private void append(PropertyExistence constraint) {
	}

    private void append(SameNode constraint) {
	}

    private void append(Column[] columns) {
	}

    private void append(Source source) {
	}

    private void append(Join join) {
	}

    private void append(JoinCondition joinCondition) {
	}

    private void append(EquiJoinCondition condition) {
	}

    private void append(ChildNodeJoinCondition condition) {
	}

    private void append(DescendantNodeJoinCondition condition) {
	}

    private void append(SameNodeJoinCondition condition) {
	}

    private void appendPath(String path) {
	}

    private void appendJoinType(String joinType) {
	}

    private void append(Selector selector) {
	}

    private void appendName(String name) {
	}

    private static boolean isSimpleName(String name) {
		return false;
	}

    private void append(String s) {
	}

}
