/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.core.config;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Properties;

import javax.naming.Context;

/**
 * This class contains list of definitions for {@code DataSource} instances.
 */
public class DataSourceConfig {

    public static final String DRIVER = "driver";

    public static final String URL = "url";

    public static final String USER = "user";

    public static final String PASSWORD = "password";

    public static final String DB_TYPE = "databaseType";

    public static final String VALIDATION_QUERY = "validationQuery";

    public static final String MAX_POOL_SIZE = "maxPoolSize";

    private final List<DataSourceDefinition> defs = new ArrayList<DataSourceDefinition>();

    /**
     * Adds a DataSourceDefinition from the given properties.
     * 
     * @param props the properties (key and values must be strings)
     * @throws ConfigurationException on error
     */
    public void addDataSourceDefinition(String name, Properties props) throws ConfigurationException {
	}

    /**
     * @return the unmodifiable list of the current {@link DataSourceDefinition}s
     */
    public List<DataSourceDefinition> getDefinitions() {
		return null;
	}

    /**
     * The definition of a DataSource. 
     */
    public static final class DataSourceDefinition {

        private static final List<String> allPropNames =
            Arrays.asList(DRIVER, URL, USER, PASSWORD, DB_TYPE, VALIDATION_QUERY, MAX_POOL_SIZE);

        private static final List<String> allJndiPropNames =
            Arrays.asList(DRIVER, URL, USER, PASSWORD, DB_TYPE);

        private final String logicalName;

        private final String driver;

        private final String url;

        private final String user;

        private final String password;

        private final String dbType;

        private final String validationQuery;

        private final int maxPoolSize;

        /**
         * Creates a DataSourceDefinition from the given properties and 
         * throws a {@link ConfigurationException} when the set of properties does not
         * satisfy some validity constraints.
         * 
         * @param name the logical name of the data source
         * @param props the properties (string keys and values)
         * @throws ConfigurationException on error
         */
        public DataSourceDefinition(String name, Properties props) throws ConfigurationException {
            this.logicalName = name;
            this.driver = (String) props.getProperty(DRIVER);
            this.url = (String) props.getProperty(URL);
            this.user = (String) props.getProperty(USER);
            this.password = (String) props.getProperty(PASSWORD);
            this.dbType = (String) props.getProperty(DB_TYPE);
            this.validationQuery = (String) props.getProperty(VALIDATION_QUERY);
            try {
                this.maxPoolSize = Integer.parseInt((String) props.getProperty(MAX_POOL_SIZE, "-1"));
            } catch (NumberFormatException e) {
                throw new ConfigurationException("failed to parse " + MAX_POOL_SIZE
                        + " property for DataSource " + logicalName);
            }
            verify(props);
        }

        private void verify(Properties props) throws ConfigurationException {
		}

        private boolean isJndiConfig() throws ConfigurationException {
			return false;
		}

        /**
         * @return the logicalName
         */
        public String getLogicalName() {
			return null;
		}

        /**
         * @return the driver
         */
        public String getDriver() {
			return null;
		}

        /**
         * @return the url
         */
        public String getUrl() {
			return null;
		}

        /**
         * @return the user
         */
        public String getUser() {
			return null;
		}

        /**
         * @return the dbType
         */
        public String getDbType() {
			return null;
		}

        /**
         * @return the password
         */
        public String getPassword() {
			return null;
		}

        /**
         * @return the validationQuery
         */
        public String getValidationQuery() {
			return null;
		}

        /**
         * @return the maxPoolSize
         */
        public int getMaxPoolSize() {
			return 0;
		}
    }
}
