/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.jackrabbit.oak.plugins.blob.datastore;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import javax.annotation.Nonnull;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.io.Closeables;
import com.google.common.io.Files;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.io.filefilter.FileFilterUtils;
import org.apache.jackrabbit.core.data.DataIdentifier;
import org.apache.jackrabbit.core.data.DataRecord;
import org.apache.jackrabbit.core.data.DataStoreException;
import org.apache.jackrabbit.core.data.LazyFileInputStream;
import org.apache.jackrabbit.oak.spi.blob.AbstractDataRecord;
import org.apache.jackrabbit.oak.spi.blob.AbstractSharedBackend;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.apache.commons.io.FilenameUtils.normalizeNoEndSeparator;

/**
 */
public class FSBackend extends AbstractSharedBackend {
    private static final Logger LOG = LoggerFactory.getLogger(FSBackend.class);

    public static final String FS_BACKEND_PATH = "fsBackendPath";
    /**
     * The maximum last modified time resolution of the file system.
     */
    private static final int ACCESS_TIME_RESOLUTION = 2000;

    private Properties properties;

    private String fsPath;

    private File fsPathDir;

    @Override
    public void init() throws DataStoreException {
	}

    @Override
    public InputStream read(DataIdentifier identifier) throws DataStoreException {
		return null;
	}

    @Override
    public void write(DataIdentifier identifier, File file) throws DataStoreException {
	}

    @Override
    public DataRecord getRecord(DataIdentifier identifier) throws DataStoreException {
		return null;
	}

    @Override
    public Iterator<DataIdentifier> getAllIdentifiers() throws DataStoreException {
		return null;
	}

    @Override
    public boolean exists(DataIdentifier identifier) throws DataStoreException {
		return false;
	}

    @Override
    public void deleteRecord(DataIdentifier identifier) throws DataStoreException {
	}

    @Override
    public void addMetadataRecord(InputStream input, String name)
        throws DataStoreException {
		}

    @Override
    public void addMetadataRecord(File input, String name) throws DataStoreException {
	}

    @Override
    public DataRecord getMetadataRecord(String name) {
		return null;
	}

    @Override
    public List<DataRecord> getAllMetadataRecords(String prefix) {
		return null;
	}

    @Override
    public boolean deleteMetadataRecord(String name) {
		return false;
	}

    @Override
    public void deleteAllMetadataRecords(String prefix) {
	}

    @Override
    public Iterator<DataRecord> getAllRecords() {
		return null;
	}


    @Override
    public void close() throws DataStoreException {
	}

    @Override
    public byte[] getOrCreateReferenceKey() throws DataStoreException {
		return null;
	}

    /*----------------------------------- Helper Methods-- -------------------------------------**/

    /**
     * Returns the identified file. This method implements the pattern used to
     * avoid problems with too many files in a single directory.
     * <p>
     * No sanity checks are performed on the given identifier.
     *
     * @param identifier data identifier
     * @return identified file
     */
    private static File getFile(DataIdentifier identifier, File root) {
		return null;
	}

    /**
     * Get the last modified date of a file.
     *
     * @param file the file
     * @return the last modified date
     * @throws DataStoreException if reading fails
     */
    private static long getLastModified(File file) throws DataStoreException {
		return 0L;
	}

    /**
     * Set the last modified date of a file, if the file is writable.
     *
     * @param file the file
     * @param time the new last modified date
     * @throws DataStoreException if the file is writable but modifying the date
     *                            fails
     */
    private static void setLastModified(File file, long time) throws DataStoreException {
	}

    private void deleteEmptyParentDirs(File file) {
	}

    /*--------------------------------- Gettters & Setters -------------------------------------**/

    /**
     * Properties used to configure the backend. These are mandatorily to be provided explicitly
     * before calling {{@link #init()} is invoked.
     *
     * @param properties to configure Backend
     */
    public void setProperties(Properties properties) {
	}

    /*-------------------------------- Inner classes -------------------------------------------**/

    /**
     * FSBackendDataRecord which lazily retrieves the input stream of the record.
     */
    class FSBackendDataRecord extends AbstractDataRecord {
        private long length;
        private long lastModified;
        private File file;

        public FSBackendDataRecord(AbstractSharedBackend backend,
            @Nonnull DataIdentifier identifier, @Nonnull File file) {
            super(backend, identifier);
            this.file = file;
            this.length = file.length();
            this.lastModified = file.lastModified();
        }

        @Override public long getLength() throws DataStoreException {
			return 0L;
		}

        @Override public InputStream getStream() throws DataStoreException {
			return null;
		}

        @Override public long getLastModified() {
			return 0L;
		}

        @Override public String toString() {
			return null;
		}
    }
}
