/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.plugins.value;

import java.io.InputStream;
import java.math.BigDecimal;
import java.util.Calendar;

import javax.jcr.Binary;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;

/**
 * Instances of this class represent a {@code Value} which couldn't be retrieved.
 * All its accessors throw a {@code RepositoryException}.
 */
class ErrorValue implements Value {
    private final Exception exception;
    private final int type;

    ErrorValue(Exception exception, int type) {
        this.exception = exception;
        this.type = type;
    }

    ErrorValue(RepositoryException e) {
        this(e, PropertyType.UNDEFINED);
    }

    @Override
    public String getString() throws RepositoryException {
		return null;
	}

    @Override
    public InputStream getStream() throws RepositoryException {
		return null;
	}

    @Override
    public Binary getBinary() throws RepositoryException {
		return null;
	}

    @Override
    public long getLong() throws RepositoryException {
		return 0L;
	}

    @Override
    public double getDouble() throws RepositoryException {
		return 0.0d;
	}

    @Override
    public BigDecimal getDecimal() throws RepositoryException {
		return null;
	}

    @Override
    public Calendar getDate() throws RepositoryException {
		return null;
	}

    @Override
    public boolean getBoolean() throws RepositoryException {
		return false;
	}

    @Override
    public int getType() {
		return 0;
	}

    private RepositoryException createException() {
		return null;
	}

    @Override
    public String toString() {
		return null;
	}
}
