/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.client;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.rmi.RemoteException;

import javax.jcr.NamespaceRegistry;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Workspace;
import javax.jcr.lock.LockManager;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.observation.ObservationManager;
import javax.jcr.query.QueryManager;
import javax.jcr.version.Version;
import javax.jcr.version.VersionManager;

import org.apache.jackrabbit.rmi.remote.RemoteNamespaceRegistry;
import org.apache.jackrabbit.rmi.remote.RemoteNodeTypeManager;
import org.apache.jackrabbit.rmi.remote.RemoteQueryManager;
import org.apache.jackrabbit.rmi.remote.RemoteWorkspace;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * Local adapter for the JCR-RMI {@link RemoteWorkspace RemoteWorkspace}
 * interface. This class makes a remote workspace locally available using
 * the JCR {@link Workspace Workspace} interface.
 *
 * @see javax.jcr.Workspace
 * @see org.apache.jackrabbit.rmi.remote.RemoteWorkspace
 */
public class ClientWorkspace extends ClientObject implements Workspace {

    /** The current session. */
    private Session session;

    /** The adapted remote workspace. */
    private RemoteWorkspace remote;

    /**
     * The adapted observation manager of this workspace. This field is set on
     * the first call to the {@link #getObservationManager()()} method assuming,
     * that the observation manager instance is not changing during the lifetime
     * of a workspace instance, that is, each call to the server-side
     * <code>Workspace.getObservationManager()</code> allways returns the same
     * object.
     */
    private ObservationManager observationManager;

    private LockManager lockManager;

    private VersionManager versionManager;

    /**
     * Creates a client adapter for the given remote workspace.
     *
     * @param session current session
     * @param remote remote workspace
     * @param factory local adapter factory
     */
    public ClientWorkspace(
            Session session, RemoteWorkspace remote,
            LocalAdapterFactory factory) {
        super(factory);
        this.session = session;
        this.remote = remote;
    }

    /**
     * Returns the current session without contacting the remote workspace.
     *
     * {@inheritDoc}
     */
    public Session getSession() {
		return null;
	}

    /** {@inheritDoc} */
    public String getName() {
		return null;
	}

    /** {@inheritDoc} */
    public void copy(String from, String to) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public void copy(String workspace, String from, String to)
            throws RepositoryException {
			}

    /** {@inheritDoc} */
    public void move(String from, String to) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public QueryManager getQueryManager() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public NamespaceRegistry getNamespaceRegistry() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public NodeTypeManager getNodeTypeManager() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public ObservationManager getObservationManager()
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public void clone(
            String workspace, String src, String dst, boolean removeExisting)
            throws RepositoryException {
			}

    /** {@inheritDoc} */
    public String[] getAccessibleWorkspaceNames() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public ContentHandler getImportContentHandler(
            final String path, final int mode) throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public void importXML(String path, InputStream xml, int uuidBehaviour)
            throws IOException, RepositoryException {
			}

    /** {@inheritDoc} */
    public void restore(Version[] versions, boolean removeExisting)
            throws RepositoryException {
			}

    public void createWorkspace(String name) throws RepositoryException {
	}

    public void createWorkspace(String name, String srcWorkspace)
            throws RepositoryException {
			}

    public void deleteWorkspace(String name) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public LockManager getLockManager() throws RepositoryException {
		return null;
	}

    public VersionManager getVersionManager() throws RepositoryException {
		return null;
	}

}
