/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.value;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.jcr.Binary;
import javax.jcr.Node;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFactory;
import javax.jcr.ValueFormatException;

/**
 * The <code>SerialValueFactory</code> class is used in the RMI infrastructure
 * to create serializable <code>Value</code> instances on the client side.
 * <p>
 * This class works in conjunction with the implementations of the
 * <code>javax.jcr.Value</code> interface found in this package.
 * <p>
 * This class may be extended to overwrite any of the
 * <code>createXXXValue</code> methods to create instances of the respective
 * type of {@link Value} implementation. The
 * methods of the <code>ValueFactory</code> interface are declared final to
 * guard against breaking the rules.
 */
public class SerialValueFactory implements ValueFactory {

    /** The singleton value factory instance */
    private static final SerialValueFactory INSTANCE = new SerialValueFactory();

    /**
     * Returns the <code>ValueFactory</code> instance, which currently is a
     * singleton instance of this class.
     * <p>
     * Future revisions will support some kind of configuration to specify
     * which concrete class should be used.
     */
    public static final SerialValueFactory getInstance() {
		return null;
	}

    /**
     * Utility method for decorating an array of values. The returned array will
     * contain serializable value decorators for all the given values. Note that
     * the contents of the original values will only be copied when the
     * decorators are serialized.
     * <p>
     * If the given array is <code>null</code>, then an empty array is
     * returned.
     *
     * @param values the values to be decorated
     * @return array of decorated values
     * @throws RepositoryException if the values can not be serialized
     */
    public static Value[] makeSerialValueArray(Value[] values)
            throws RepositoryException {
				return null;
			}

    /**
     * Utility method for decorating a value. Note that the contents of the
     * original values will only be copied when the decorators are serialized.
     * Null referenced and already serializable values are passed as-is.
     *
     * @param value the value to be decorated, or <code>null</code>
     * @return the decorated value, or <code>null</code>
     * @throws RepositoryException if the value can not be serialized
     */
    public static Value makeSerialValue(Value value) throws RepositoryException {
		return null;
	}

    /**
     * Utility method for converting an array of strings to serializable
     * string values.
     * <p>
     * If the given array is <code>null</code>, then an empty array is
     * returned.
     *
     * @param values the string array
     * @return array of string values
     */
    public static Value[] makeSerialValueArray(String[] values) {
		return null;
	}

    /**
     * Default constructor only visible to extensions of this class. See
     * class comments for details.
     */
    protected SerialValueFactory() {
    }

    /** {@inheritDoc} */
    public Value createValue(String value) {
		return null;
	}

    /** {@inheritDoc} */
    public final Value createValue(String value, int type)
            throws ValueFormatException {
				return null;
			}

    private Value createValue(Value value) throws RepositoryException {
		return null;
	}

    private Value createValue(Value value, int type)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public final Value createValue(long value) {
		return null;
	}

    /** {@inheritDoc} */
    public final Value createValue(double value) {
		return null;
	}

    /** {@inheritDoc} */
    public final Value createValue(boolean value) {
		return null;
	}

    /** {@inheritDoc} */
    public Value createValue(BigDecimal value) {
		return null;
	}

    /** {@inheritDoc} */
    public final Value createValue(Calendar value) {
		return null;
	}

    /** {@inheritDoc} */
    public final Value createValue(InputStream value) {
		return null;
	}

    /** {@inheritDoc} */
    public final Value createValue(Node value) throws RepositoryException {
		return null;
	}

    public Binary createBinary(InputStream stream) throws RepositoryException {
		return null;
	}

    public Value createValue(Binary value) {
		return null;
	}

    public Value createValue(Node value, boolean weak)
            throws RepositoryException {
				return null;
			}

}
