/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.value;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.TimeZone;

import javax.jcr.PropertyType;
import javax.jcr.ValueFormatException;

/**
 * String value.
 */
class StringValue extends AbstractValue {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 220963478492833703L;

    /** The string value */
    private final String value;

    /**
     * Creates an instance for the given string <code>value</code>.
     */
    public StringValue(String value) {
        this.value = value;
    }

    /**
     * Returns {@link PropertyType#STRING}.
     */
    public int getType() {
		return 0;
	}

    /**
     * The string is converted using {@link Boolean#valueOf(String)}.
     */
    @Override
    public boolean getBoolean() {
		return false;
	}

    /**
     * If the string is in the format described in
     * {@link DateValue#getString()}, it is converted directly, otherwise
     * a {@link ValueFormatException} is thrown.
     */
    @Override
    public Calendar getDate() throws ValueFormatException {
		return null;
	}

    /**
     * The string is converted using the constructor
     * {@link BigDecimal#BigDecimal(String)}.
     */
    @Override
    public BigDecimal getDecimal() throws ValueFormatException {
		return null;
	}


    /**
     * The string is converted using {@link Double#valueOf(String)}.
     */
    @Override
    public double getDouble() throws ValueFormatException {
		return 0.0d;
	}

    /**
     * The string is converted using {@link Long#valueOf(String)}.
     */
    @Override
    public long getLong() throws ValueFormatException {
		return 0L;
	}

    /**
     * Returns the string value.
     */
    public String getString() {
		return null;
	}

}
