/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.name;

import javax.jcr.RepositoryException;

import org.apache.jackrabbit.spi.Path;

abstract class RelativePath extends AbstractPath {

    /** Serial version UID */
    private static final long serialVersionUID = 5707676677044863127L;

    protected final Path parent;

    private final boolean absolute;

    private final boolean identifier;

    private final int depth;

    private final int length;

    protected RelativePath(Path parent) {
        this.parent = parent;
        if (parent != null) {
            this.absolute = parent.isAbsolute();
            this.identifier = parent.isIdentifierBased();
            this.depth = parent.getDepth() + getDepthModifier();
            this.length = parent.getLength() + 1;
        } else {
            this.absolute = false;
            this.identifier = false;
            this.depth = getDepthModifier();
            this.length = 1;
        }
    }

    protected abstract int getDepthModifier() {
		return 0;
	}

    protected abstract Path getParent() throws RepositoryException {
		return null;
	}

    protected abstract String getElementString() {
		return null;
	}

    public final boolean isIdentifierBased() {
		return false;
	}

    public final boolean isAbsolute() {
		return false;
	}

    public final Path getAncestor(int degree) throws RepositoryException {
		return null;
	}

    public final int getAncestorCount() {
		return 0;
	}

    public final int getDepth() {
		return 0;
	}

    public final int getLength() {
		return 0;
	}

    public final Path subPath(int from, int to) {
		return null;
	}

    public final Element[] getElements() {
		return null;
	}

    /**
     * Returns the first elements of this path.
     *
     * @return first elements of this path, or <code>null</code>
     */
    @Override
    public Path getFirstElements() {
		return null;
	}

    public String getString() {
		return null;
	}

    //--------------------------------------------------------------< Object >

    public boolean equals(Object that) {
		return false;
	}

    public int hashCode() {
		return 0;
	}

}
