/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.query;

import java.util.Arrays;

import javax.jcr.RepositoryException;

import org.apache.jackrabbit.spi.Name;
import org.apache.jackrabbit.spi.Path;

/**
 * Utility class to dump a {@link QueryNode} tree to a StringBuffer.
 */
public class QueryTreeDump implements QueryNodeVisitor {

    /**
     * Current indentation level
     */
    private int indent;

    /**
     * Padding array filled with spaces
     */
    private static char[] PADDING = new char[255];

    /**
     * The padding character: whitespace.
     */
    private static final char PADDING_CHAR = ' ';

    static {
        Arrays.fill(PADDING, PADDING_CHAR);
    }

    /**
     * Dumps the node tree to buffer.
     * @param node the root node.
     * @param buffer where to dump the tree.
     * @throws RepositoryException
     */
    private QueryTreeDump(QueryNode node, StringBuffer buffer) throws RepositoryException {
        node.accept(this, buffer);
    }

    /**
     * Dumps a query node tree to the string <code>buffer</code>.
     * @param node the root node of a query tree.
     * @param buffer a string buffer where to dump the tree structure.
     * @throws RepositoryException
     */
    public static void dump(QueryNode node, StringBuffer buffer) throws RepositoryException {
	}

    public Object visit(QueryRootNode node, Object data) throws RepositoryException {
		return null;
	}

    public Object visit(OrQueryNode node, Object data) throws RepositoryException {
		return null;
	}

    public Object visit(AndQueryNode node, Object data) throws RepositoryException {
		return null;
	}

    public Object visit(NotQueryNode node, Object data) throws RepositoryException {
		return null;
	}

    public Object visit(ExactQueryNode node, Object data) {
		return null;
	}

    public Object visit(NodeTypeQueryNode node, Object data) {
		return null;
	}

    public Object visit(TextsearchQueryNode node, Object data) {
		return null;
	}

    public Object visit(PathQueryNode node, Object data) throws RepositoryException {
		return null;
	}

    public Object visit(LocationStepQueryNode node, Object data) throws RepositoryException {
		return null;
	}

    public Object visit(RelationQueryNode node, Object data) throws RepositoryException {
		return null;
	}

    public Object visit(OrderQueryNode node, Object data) {
		return null;
	}

    public Object visit(DerefQueryNode node, Object data) throws RepositoryException {
		return null;
	}

    public Object visit(PropertyFunctionQueryNode node, Object data) {
		return null;
	}

    private void traverse(QueryNode[] node, StringBuffer buffer) throws RepositoryException {
	}

    /**
     * Appends the relative path to the <code>buffer</code> using '/' as the
     * delimiter for path elements.
     *
     * @param relPath a relative path.
     * @param buffer the buffer where to append the path.
     */
    private static void appendPath(Path relPath, StringBuffer buffer) {
	}
}
