/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.value;

import java.util.Calendar;
import java.util.UUID;
import java.math.BigDecimal;
import java.net.URI;
import java.io.UnsupportedEncodingException;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.ValueFormatException;

import org.apache.jackrabbit.spi.Name;
import org.apache.jackrabbit.spi.NameFactory;
import org.apache.jackrabbit.spi.PathFactory;
import org.apache.jackrabbit.spi.QPropertyDefinition;
import org.apache.jackrabbit.spi.QValue;
import org.apache.jackrabbit.spi.QValueFactory;
import org.apache.jackrabbit.spi.Path;
import org.apache.jackrabbit.spi.commons.name.NameConstants;
import org.apache.jackrabbit.spi.commons.name.NameFactoryImpl;
import org.apache.jackrabbit.spi.commons.name.PathFactoryImpl;
import org.apache.jackrabbit.util.ISO8601;

/**
 * <code>AbstractQValueFactory</code>...
 */
public abstract class AbstractQValueFactory implements QValueFactory {

    /**
     * the default encoding
     */
    public static final String DEFAULT_ENCODING = "UTF-8";

    protected static final PathFactory PATH_FACTORY = PathFactoryImpl.getInstance();
    protected static final NameFactory NAME_FACTORY = NameFactoryImpl.getInstance();


    //------------------------------------------------------< QValueFactory >---
    /**
     * @see QValueFactory#computeAutoValues(org.apache.jackrabbit.spi.QPropertyDefinition)
     */
    public QValue[] computeAutoValues(QPropertyDefinition propertyDefinition) throws RepositoryException {
		return null;
	}

    /**
     * @see QValueFactory#create(String, int)
     */
    public QValue create(String value, int type) throws RepositoryException {
		return null;
	}

    /**
     * @see QValueFactory#create(Calendar)
     */
    public QValue create(Calendar value) throws RepositoryException {
		return null;
	}

    /**
     * @see QValueFactory#create(double)
     */
    public QValue create(double value) throws RepositoryException {
		return null;
	}

    /**
     * @see QValueFactory#create(long)
     */
    public QValue create(long value) throws RepositoryException {
		return null;
	}

    /**
     * @see QValueFactory#create(boolean)
     */
    public QValue create(boolean value) throws RepositoryException {
		return null;
	}

    /**
     * @see QValueFactory#create(Name)
     */
    public QValue create(Name value) throws RepositoryException {
		return null;
	}

    /**
     * @see QValueFactory#create(Path)
     */
    public QValue create(Path value) throws RepositoryException {
		return null;
	}

    /**
     * @see QValueFactory#create(URI)
     */
    public QValue create(URI value) throws RepositoryException {
		return null;
	}

    /**
     * @see QValueFactory#create(URI)
     */
    public QValue create(BigDecimal value) throws RepositoryException {
		return null;
	}

    /**
     * Creates a new QValue of type STRING.
     *
     * @param value the string value.
     * @return a new QValue.
     */
    protected QValue createString(String value) {
		return null;
	}

    /**
     * Creates a new QValue of type REFERENCE or WEAKREFERENCE.
     *
     * @param ref the reference value.
     * @param weak whether the reference is weak.
     * @return a new QValue.
     */
    protected QValue createReference(String ref, boolean weak) {
		return null;
	}
}