/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.webdav.security;

import org.apache.jackrabbit.webdav.DavConstants;
import org.apache.jackrabbit.webdav.DavException;
import org.apache.jackrabbit.webdav.DavServletResponse;
import org.apache.jackrabbit.webdav.property.DavPropertyName;
import org.apache.jackrabbit.webdav.xml.DomUtil;
import org.apache.jackrabbit.webdav.xml.XmlSerializable;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.util.HashMap;
import java.util.Map;

/**
 * <code>Principal</code> encapsulates the DAV:principal element which identifies
 * the principal to which this ACE applies.
 * RFC 3744 defines the following structure for this element:
 * <pre>
 * &lt;!ELEMENT principal (href | all | authenticated | unauthenticated | property | self)&gt;
 * </pre>
 */
public class Principal implements XmlSerializable, SecurityConstants {

    public static final String XML_PRINCIPAL = "principal";

    private static final String XML_ALL = "all";
    private static final String XML_AUTHENTICATED  = "authenticated";
    private static final String XML_UNAUTHENTICATED = "unauthenticated";
    private static final String XML_SELF = "self";
    private static final String XML_PROPERTY = "property";

    private static final int TYPE_ALL = 0;
    private static final int TYPE_AUTHENTICATED = 1;
    private static final int TYPE_UNAUTHENTICATED = 2;
    private static final int TYPE_SELF = 3;
    private static final int TYPE_PROPERTY = 4;
    private static final int TYPE_HREF = 5;

    private static final Principal ALL_PRINCIPAL = new Principal(TYPE_ALL);
    private static final Principal AUTHENTICATED_PRINCIPAL = new Principal(TYPE_AUTHENTICATED);
    private static final Principal UNAUTHENTICATED_PRINCIPAL = new Principal(TYPE_UNAUTHENTICATED);
    private static final Principal SELF_PRINCIPAL = new Principal(TYPE_SELF);

    private static final Map<DavPropertyName, Principal> PROP_PRINCIPALS = new HashMap<DavPropertyName, Principal>();

    private final int type;
    private DavPropertyName propertyName;
    private String href;

    private Principal(int type) {
        this.type = type;
    }

    private Principal(DavPropertyName propertyName) {
        this.type = TYPE_PROPERTY;
        this.propertyName = propertyName;
    }

    private Principal(String href) {
        this.type = TYPE_HREF;
        this.href = href;
    }

    /**
     * @return href if this Principal is a href-typed principal, <code>null</code>
     * otherwise.
     */
    public String getHref() {
		return null;
	}

    /**
     * @return propertyName if this Principal is a property-principal,
     * <code>null</code> otherwise.
     */
    public DavPropertyName getPropertyName() {
		return null;
	}

    /**
     * @see XmlSerializable#toXml(Document)
     */
    public Element toXml(Document document) {
		return null;
	}

    public static Principal getAllPrincipal() {
		return null;
	}

    public static Principal getAuthenticatedPrincipal() {
		return null;
	}

    public static Principal getUnauthenticatedPrincipal() {
		return null;
	}

    public static Principal getSelfPrincipal() {
		return null;
	}

    public static Principal getPropertyPrincipal(DavPropertyName propertyName) {
		return null;
	}

    public static Principal getHrefPrincipal(String href) {
		return null;
	}

    public static Principal createFromXml(Element principalElement) throws DavException {
		return null;
	}
}
