/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.discovery.base.connectors.announcement;

import org.apache.sling.discovery.base.connectors.BaseConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * With SLING-3389 the Announcement itself doesn't use the created
 * (ie timeout) field anymore (it still has it currently for backwards
 * compatibility on the wire-level) - hence that's why there's this
 * small in-memory wrapper object which contains an Announcement and 
 * carries a lastHeartbeat property.
 */
public class CachedAnnouncement {
    
    private final static Logger logger = LoggerFactory.getLogger(CachedAnnouncement.class);

    private long lastPing = System.currentTimeMillis();

    private final Announcement announcement;
    
    private long firstPing = System.currentTimeMillis();

    private long backoffIntervalSeconds = -1;

    private final BaseConfig config;
    
    CachedAnnouncement(final Announcement announcement, final BaseConfig config) {
        this.announcement = announcement;
        this.config = config;
    }
    
    private long getConfiguredConnectorTimeout() {
		return 0L;
	}
    
    private long getConfiguredConnectorInterval() {
		return 0L;
	}

    public final boolean hasExpired() {
		return false;
	}
    
    public final long getLastPing() {
		return 0L;
	}
    
    public final long getFirstPing() {
		return 0L;
	}
    
    /** Returns the second until the next heartbeat is expected, otherwise the timeout will hit **/
    public final long getSecondsUntilTimeout() {
		return 0L;
	}
    
    
    private final long getEffectiveHeartbeatTimeout() {
		return 0L;
	}

    /** Registers a heartbeat event, and returns the new resulting backoff interval -
     * or 0 if no backoff is applicable yet.
     * @param incomingAnnouncement 
     * @return the new resulting backoff interval -
     * or 0 if no backoff is applicable yet.
     */
    final long registerPing(Announcement incomingAnnouncement, BaseConfig config) {
		return 0L;
	}

    public final Announcement getAnnouncement() {
		return null;
	}

}
