/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to You under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apache.sling.engine.jmx;

import org.osgi.annotation.versioning.ProviderType;

/**
 * This is the management interface for the SlingRequestProcessor.
 */
@ProviderType
public interface RequestProcessorMBean {

    /**
     * Returns the number of requests collected since last resetting the
     * statistics.
     *
     * @return Number of requests
     * @see #resetStatistics()
     */
    long getRequestsCount() {
		return 0L;
	}

    /**
     * Returns the time in milliseconds used by the longest request since last
     * resetting the statistics.
     *
     * @return Max request duration
     * @see #resetStatistics()
     */
    long getMaxRequestDurationMsec() {
		return 0L;
	}

    /**
     * Returns the time in milliseconds used by the shortest request since last
     * resetting the statistics.
     *
     * @return Min request duration
     * @see #resetStatistics()
     */
    long getMinRequestDurationMsec() {
		return 0L;
	}

    /**
     * Returns the mean request processing time in milliseconds since resetting
     * the statistics.
     *
     * @return Mean request duration
     * @see #resetStatistics()
     */
    double getMeanRequestDurationMsec() {
		return 0.0d;
	}

    /**
     * Returns the standard deviation of requests since resetting the
     * statistics. If zero or one requests have been collected only, this method
     * returns zero.
     *
     * @return Standard deviation
     * @see #resetStatistics()
     */
    double getStandardDeviationDurationMsec() {
		return 0.0d;
	}

    /**
     * Returns the maximum peak recursive execution depth since last
     * resetting the statistics.
     *
     * @return Max peak recursion depth
     * @see #resetStatistics()
     */
    int getMaxPeakRecursionDepth() {
		return 0;
	}


    /**
     * Returns the minimal peak recursive execution depth since last
     * resetting the statistics.
     *
     * @return Min peak recursion depth
     * @see #resetStatistics()
     */
    int getMinPeakRecursionDepth() {
		return 0;
	}


    /**
     * Returns the mean peak recursive execution depth since last
     * resetting the statistics.
     *
     * @return Mean peak recursion depth
     * @see #resetStatistics()
     */
    double getMeanPeakRecursionDepth() {
		return 0.0d;
	}


    /**
     * Returns the standard deviation of peak recursive execution depth since last
     * resetting the statistics.
     *
     * @return Standard deviation of peak recursion depth
     * @see #resetStatistics()
     */
    double getStandardDeviationPeakRecursionDepth() {
		return 0.0d;
	}

    /**
     * Returns the maximum servlet call count since last
     * resetting the statistics.
     *
     * @return Max servlet count call
     * @see #resetStatistics()
     */
    int getMaxServletCallCount() {
		return 0;
	}


    /**
     * Returns the minimum servlet call count since last
     * resetting the statistics.
     *
     * @return Min servlet count call
     * @see #resetStatistics()
     */
    int getMinServletCallCount() {
		return 0;
	}

    /**
     * Returns the mean servlet call count since last
     * resetting the statistics.
     *
     * @return Mean servlet count call
     * @see #resetStatistics()
     */
    double getMeanServletCallCount() {
		return 0.0d;
	}

    /**
     * Returns the standard deviation servlet call counts since last
     * resetting the statistics.
     *
     * @return Mean standard deviation for servlet count call
     * @see #resetStatistics()
     */
    double getStandardDeviationServletCallCount() {
		return 0.0d;
	}

    /**
     * Resets all statistics values and restarts from zero.
     */
    void resetStatistics() {
	}

}
