/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.sling.scripting.jsp.jasper.runtime;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.el.ELContext;
import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspContext;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.el.ELException;
import javax.servlet.jsp.el.ExpressionEvaluator;
import javax.servlet.jsp.el.VariableResolver;
import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.VariableInfo;

import org.apache.sling.scripting.jsp.jasper.compiler.Localizer;
import org.apache.sling.scripting.jsp.jasper.util.Enumerator;

/**
 * Implementation of a JSP Context Wrapper.
 * 
 * The JSP Context Wrapper is a JspContext created and maintained by a tag
 * handler implementation. It wraps the Invoking JSP Context, that is, the
 * JspContext instance passed to the tag handler by the invoking page via
 * setJspContext().
 * 
 * @author Kin-man Chung
 * @author Jan Luehe
 * @author Jacob Hookom
 */
public class JspContextWrapper extends PageContext implements VariableResolver {

	// Invoking JSP context
	private PageContext invokingJspCtxt;

	private transient HashMap<String, Object> pageAttributes;

	// ArrayList of NESTED scripting variables
	private ArrayList nestedVars;

	// ArrayList of AT_BEGIN scripting variables
	private ArrayList atBeginVars;

	// ArrayList of AT_END scripting variables
	private ArrayList atEndVars;

	private Map aliases;

	private HashMap<String, Object> originalNestedVars;

	public JspContextWrapper(JspContext jspContext, ArrayList nestedVars,
			ArrayList atBeginVars, ArrayList atEndVars, Map aliases) {
		this.invokingJspCtxt = (PageContext) jspContext;
		this.nestedVars = nestedVars;
		this.atBeginVars = atBeginVars;
		this.atEndVars = atEndVars;
		this.pageAttributes = new HashMap<String, Object>(16);
		this.aliases = aliases;

		if (nestedVars != null) {
			this.originalNestedVars = new HashMap<String, Object>(nestedVars.size());
		}
		syncBeginTagFile();
	}

	public void initialize(Servlet servlet, ServletRequest request,
			ServletResponse response, String errorPageURL,
			boolean needsSession, int bufferSize, boolean autoFlush)
			throws IOException, IllegalStateException, IllegalArgumentException {
			}

	public Object getAttribute(String name) {
		return null;
	}

	public Object getAttribute(String name, int scope) {
		return null;
	}

	public void setAttribute(String name, Object value) {
	}

	public void setAttribute(String name, Object value, int scope) {
	}

	public Object findAttribute(String name) {
		return null;
	}

	public void removeAttribute(String name) {
	}

	public void removeAttribute(String name, int scope) {
	}

	public int getAttributesScope(String name) {
		return 0;
	}

	public Enumeration<String> getAttributeNamesInScope(int scope) {
		return null;
	}

	public void release() {
	}

	public JspWriter getOut() {
		return null;
	}

	public HttpSession getSession() {
		return null;
	}

	public Object getPage() {
		return null;
	}

	public ServletRequest getRequest() {
		return null;
	}

	public ServletResponse getResponse() {
		return null;
	}

	public Exception getException() {
		return null;
	}

	public ServletConfig getServletConfig() {
		return null;
	}

	public ServletContext getServletContext() {
		return null;
	}

	public void forward(String relativeUrlPath) throws ServletException,
			IOException {
			}

	public void include(String relativeUrlPath) throws ServletException,
			IOException {
			}

	public void include(String relativeUrlPath, boolean flush)
			throws ServletException, IOException {
			}

	public VariableResolver getVariableResolver() {
		return null;
	}

	public BodyContent pushBody() {
		return null;
	}

	public JspWriter pushBody(Writer writer) {
		return null;
	}

	public JspWriter popBody() {
		return null;
	}

	public ExpressionEvaluator getExpressionEvaluator() {
		return null;
	}

	public void handlePageException(Exception ex) throws IOException,
			ServletException {
			}

	public void handlePageException(Throwable t) throws IOException,
			ServletException {
			}

	/**
	 * VariableResolver interface
	 */
	public Object resolveVariable(String pName) throws ELException {
		return null;
	}

	/**
	 * Synchronize variables at begin of tag file
	 */
	public void syncBeginTagFile() {
	}

	/**
	 * Synchronize variables before fragment invokation
	 */
	public void syncBeforeInvoke() {
	}

	/**
	 * Synchronize variables at end of tag file
	 */
	public void syncEndTagFile() {
	}

	/**
	 * Copies the variables of the given scope from the virtual page scope of
	 * this JSP context wrapper to the page scope of the invoking JSP context.
	 * 
	 * @param scope
	 *            variable scope (one of NESTED, AT_BEGIN, or AT_END)
	 */
	private void copyTagToPageScope(int scope) {
	}

	/**
	 * Saves the values of any NESTED variables that are present in the invoking
	 * JSP context, so they can later be restored.
	 */
	private void saveNestedVariables() {
	}

	/**
	 * Restores the values of any NESTED variables in the invoking JSP context.
	 */
	private void restoreNestedVariables() {
	}

	/**
	 * Checks to see if the given variable name is used as an alias, and if so,
	 * returns the variable name for which it is used as an alias.
	 * 
	 * @param varName
	 *            The variable name to check
	 * @return The variable name for which varName is used as an alias, or
	 *         varName if it is not being used as an alias
	 */
	private String findAlias(String varName) {
		return null;
	}

	//private ELContextImpl elContext;

	public ELContext getELContext() {
		return null;
	}
}
