/*******************************************************************************
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.sling.scripting.sightly.render;

import java.lang.reflect.Method;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.math.NumberUtils;
import org.apache.sling.scripting.sightly.Record;
import org.apache.sling.scripting.sightly.compiler.util.ObjectModel;

/**
 * Default abstract implementation of {@link RuntimeObjectModel}.
 */
public abstract class AbstractRuntimeObjectModel implements RuntimeObjectModel {

    /**
     * A {@link Set} that stores all the supported primitive classes.
     */
    public static final Set<Class<?>> PRIMITIVE_CLASSES = ObjectModel.PRIMITIVE_CLASSES;

    public static final String TO_STRING_METHOD = "toString";

    @Override
    public boolean isPrimitive(Object obj) {
		return false;
	}

    @Override
    public boolean isDate(Object target) {
		return false;
	}

    @Override
    public boolean isNumber(Object target) {
		return false;
	}

    @Override
    public boolean isCollection(Object target) {
		return false;
	}

    @Override
    public Object resolveProperty(Object target, Object property) {
		return null;
	}

    @Override
    public boolean toBoolean(Object object) {
		return false;
	}

    @Override
    public Number toNumber(Object object) {
		return null;
	}

    @Override
    public Date toDate(Object object) {
		return null;
	}

    @Override
    public String toString(Object target) {
		return null;
	}

    @Override
    public Collection<Object> toCollection(Object object) {
		return null;
	}

    @Override
    public Map toMap(Object object) {
		return null;
	}

    protected Object getProperty(Object target, Object propertyObj) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#toCollection(Object)}
     */
    @Deprecated
    protected Collection<Object> obtainCollection(Object obj) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#toString(Object)}
     */
    @Deprecated
    protected String objectToString(Object obj) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#collectionToString(Collection)}
     */
    @Deprecated
    protected String collectionToString(Collection<?> col) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#fromIterator(Iterator)}
     */
    @Deprecated
    protected Collection<Object> fromIterator(Iterator<Object> iterator) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#toBoolean(Object)}
     */
    @Deprecated
    protected boolean toBooleanInternal(Object obj) {
		return false;
	}

    /**
     * @deprecated see {@link ObjectModel#getIndex(Object, int)}
     */
    @Deprecated
    protected Object getIndex(Object obj, int index) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#getIndex(Object, int)}
     */
    @Deprecated
    protected Object getIndexSafe(List list, int index) {
		return null;
	}

    /**
     * @deprecated use {@link Map#get(Object)}
     */
    @Deprecated
    protected Object getMapProperty(Map map, String property) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#resolveProperty(Object, Object)}
     */
    @Deprecated
    protected Object getObjectProperty(Object obj, String property) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#getField(Object, String)}
     */
    @Deprecated
    protected static Object getField(Object obj, String property) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#invokeBeanMethod(Object, String)}
     */
    @Deprecated
    protected Object getObjectNoArgMethod(Object obj, String property) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#findBeanMethod(Class, String)}
     */
    @Deprecated
    protected static Method findMethod(Class<?> cls, String baseName) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#isMethodAllowed(Method)}
     */
    @Deprecated
    protected static boolean isMethodAllowed(Method method) {
		return false;
	}

    /**
     * @deprecated see {@link ObjectModel#findBeanMethod(Class, String)} (Class, Method)}
     */
    @Deprecated
    protected Method extractMethodInheritanceChain(Class type, Method m) {
		return null;
	}

    /**
     * @deprecated see {@link ObjectModel#findBeanMethod(Class, String)} (Class, Method)}
     */
    @Deprecated
    protected Method getClassMethod(Class<?> clazz, Method m) {
		return null;
	}

}
